@@ -14,32 +14,60 @@
 // You should have received a copy of the GNU General Public License
 // along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
 
-/**
- * @package   core_backup
- * @category  phpunit
- * @copyright 2010 onwards Eloy Lafuente (stronk7) {@link http://stronk7.com}
- * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
- */
+namespace core_backup;
+
+use backup;
+use backup_attribute;
+use backup_controller_dbops;
+use backup_final_element;
+use backup_nested_element;
+use backup_optigroup;
+use backup_optigroup_element;
+use backup_processor_exception;
+use backup_structure_processor;
+use base_element_struct_exception;
+use base_optigroup_exception;
+use base_processor;
+use memory_xml_output;
+use xml_writer;
 
 defined('MOODLE_INTERNAL') || die();
 
-// Include all the needed stuff
+// Include all the needed stuff.
 require_once(__DIR__.'/fixtures/structure_fixtures.php');
 
 global $CFG;
 require_once($CFG->dirroot . '/backup/util/xml/output/memory_xml_output.class.php');
 
-
 /**
  * Unit test case the all the backup structure classes. Note: Uses database
+ *
+ * @package   core_backup
+ * @category  test
+ * @copyright 2010 onwards Eloy Lafuente (stronk7) {@link http://stronk7.com}
+ * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
-class backup_structure_testcase extends advanced_testcase {
+final class structure_test extends \advanced_testcase {
 
-    protected $forumid;   // To store the inserted forum->id
-    protected $contextid; // Official contextid for these tests
+    /** @var int Store the inserted forum->id for use in test functions */
+    protected $forumid;
+    /** @var int Store the inserted discussion1->id for use in test functions */
+    protected $discussionid1;
+    /** @var int Store the inserted discussion2->id for use in test functions */
+    protected $discussionid2;
+    /** @var int Store the inserted post1->id for use in test functions */
+    protected $postid1;
+    /** @var int Store the inserted post2->id for use in test functions */
+    protected $postid2;
+    /** @var int Store the inserted post3->id for use in test functions */
+    protected $postid3;
+    /** @var int Store the inserted post4->id for use in test functions */
+    protected $postid4;
+    /** @var int Official contextid for these tests */
+    protected $contextid;
 
 
-    protected function setUp() {
+    protected function setUp(): void {
         parent::setUp();
 
         $this->resetAfterTest(true);
@@ -72,30 +100,30 @@ class backup_structure_testcase extends advanced_testcase {
 
         // Create two discussions
         $discussion1 = (object)array('course' => 1, 'forum' => $this->forumid, 'name' => 'd1', 'userid' => 100, 'groupid' => 200);
-        $d1id = $DB->insert_record('forum_discussions', $discussion1);
+        $this->discussionid1 = $DB->insert_record('forum_discussions', $discussion1);
         $discussion2 = (object)array('course' => 1, 'forum' => $this->forumid, 'name' => 'd2', 'userid' => 101, 'groupid' => 201);
-        $d2id = $DB->insert_record('forum_discussions', $discussion2);
+        $this->discussionid2 = $DB->insert_record('forum_discussions', $discussion2);
 
         // Create four posts
-        $post1 = (object)array('discussion' => $d1id, 'userid' => 100, 'subject' => 'p1', 'message' => 'm1');
-        $p1id = $DB->insert_record('forum_posts', $post1);
-        $post2 = (object)array('discussion' => $d1id, 'parent' => $p1id, 'userid' => 102, 'subject' => 'p2', 'message' => 'm2');
-        $p2id = $DB->insert_record('forum_posts', $post2);
-        $post3 = (object)array('discussion' => $d1id, 'parent' => $p2id, 'userid' => 103, 'subject' => 'p3', 'message' => 'm3');
-        $p3id = $DB->insert_record('forum_posts', $post3);
-        $post4 = (object)array('discussion' => $d2id, 'userid' => 101, 'subject' => 'p4', 'message' => 'm4');
-        $p4id = $DB->insert_record('forum_posts', $post4);
+        $post1 = (object)array('discussion' => $this->discussionid1, 'userid' => 100, 'subject' => 'p1', 'message' => 'm1');
+        $this->postid1 = $DB->insert_record('forum_posts', $post1);
+        $post2 = (object)array('discussion' => $this->discussionid1, 'parent' => $this->postid1, 'userid' => 102, 'subject' => 'p2', 'message' => 'm2');
+        $this->postid2 = $DB->insert_record('forum_posts', $post2);
+        $post3 = (object)array('discussion' => $this->discussionid1, 'parent' => $this->postid2, 'userid' => 103, 'subject' => 'p3', 'message' => 'm3');
+        $this->postid3 = $DB->insert_record('forum_posts', $post3);
+        $post4 = (object)array('discussion' => $this->discussionid2, 'userid' => 101, 'subject' => 'p4', 'message' => 'm4');
+        $this->postid4 = $DB->insert_record('forum_posts', $post4);
         // With two related file
         $f1_post1 = (object)array(
             'contenthash' => 'testp1', 'contextid' => $this->contextid, 'component'=>'mod_forum',
-            'filearea' => 'post', 'filename' => 'tp1', 'itemid' => $p1id,
+            'filearea' => 'post', 'filename' => 'tp1', 'itemid' => $this->postid1,
             'filesize' => 123, 'timecreated' => 0, 'timemodified' => 0,
             'pathnamehash' => 'testp1'
         );
         $DB->insert_record('files', $f1_post1);
         $f1_post2 = (object)array(
             'contenthash' => 'testp2', 'contextid' => $this->contextid, 'component'=>'mod_forum',
-            'filearea' => 'attachment', 'filename' => 'tp2', 'itemid' => $p2id,
+            'filearea' => 'attachment', 'filename' => 'tp2', 'itemid' => $this->postid2,
             'filesize' => 123, 'timecreated' => 0, 'timemodified' => 0,
             'pathnamehash' => 'testp2'
         );
@@ -103,16 +131,16 @@ class backup_structure_testcase extends advanced_testcase {
 
         // Create two ratings
         $rating1 = (object)array(
-            'contextid' => $this->contextid, 'userid' => 104, 'itemid' => $p1id, 'rating' => 2,
+            'contextid' => $this->contextid, 'userid' => 104, 'itemid' => $this->postid1, 'rating' => 2,
             'scaleid' => -1, 'timecreated' => time(), 'timemodified' => time());
         $r1id = $DB->insert_record('rating', $rating1);
         $rating2 = (object)array(
-            'contextid' => $this->contextid, 'userid' => 105, 'itemid' => $p1id, 'rating' => 3,
+            'contextid' => $this->contextid, 'userid' => 105, 'itemid' => $this->postid1, 'rating' => 3,
             'scaleid' => -1, 'timecreated' => time(), 'timemodified' => time());
         $r2id = $DB->insert_record('rating', $rating2);
 
         // Create 1 reads
-        $read1 = (object)array('userid' => 102, 'forumid' => $this->forumid, 'discussionid' => $d2id, 'postid' => $p4id);
+        $read1 = (object)array('userid' => 102, 'forumid' => $this->forumid, 'discussionid' => $this->discussionid2, 'postid' => $this->postid4);
         $DB->insert_record('forum_read', $read1);
     }
 
@@ -134,9 +162,9 @@ class backup_structure_testcase extends advanced_testcase {
                 'rsstype', 'rssarticles', 'timemodified', 'warnafter',
                 'blockafter',
                 new backup_final_element('blockperiod'),
-                new mock_skip_final_element('completiondiscussions'),
-                new mock_modify_final_element('completionreplies'),
-                new mock_final_element_interceptor('completionposts'))
+                new \mock_skip_final_element('completiondiscussions'),
+                new \mock_modify_final_element('completionreplies'),
+                new \mock_final_element_interceptor('completionposts'))
         );
         $discussions = new backup_nested_element('discussions');
         $discussion = new backup_nested_element('discussion',
@@ -195,11 +223,11 @@ class backup_structure_testcase extends advanced_testcase {
 
         // Let's add 1 optigroup with 4 elements
         $alternative1 = new backup_optigroup_element('alternative1',
-            array('name', 'value'), '../../id', 1);
+            array('name', 'value'), '../../id', $this->postid1);
         $alternative2 = new backup_optigroup_element('alternative2',
-            array('name', 'value'), backup::VAR_PARENTID, 2);
+            array('name', 'value'), backup::VAR_PARENTID, $this->postid2);
         $alternative3 = new backup_optigroup_element('alternative3',
-            array('name', 'value'), '/forum/discussions/discussion/posts/post/id', 3);
+            array('name', 'value'), '/forum/discussions/discussion/posts/post/id', $this->postid3);
         $alternative4 = new backup_optigroup_element('alternative4',
             array('forumtype', 'forumname')); // Alternative without conditions
         // Create the optigroup, adding one element
@@ -239,7 +267,7 @@ class backup_structure_testcase extends advanced_testcase {
             array(backup::VAR_PARENTID)
         );
 
-        $read->set_source_table('forum_read', array('id' => '../../id'));
+        $read->set_source_table('forum_read', array('forumid' => '../../id'));
 
         $inventeds->set_source_array(array((object)array('reason' => 'I love Moodle', 'version' => '1.0'),
             (object)array('reason' => 'I love Moodle', 'version' => '2.0'))); // 2 object array
@@ -312,9 +340,9 @@ class backup_structure_testcase extends advanced_testcase {
         $this->assertEquals($inventeds->get_counter(), 2); // Array
 
         // Perform some validations with the generated XML
-        $dom = new DomDocument();
+        $dom = new \DomDocument();
         $dom->loadXML($xo->get_allcontents());
-        $xpath = new DOMXPath($dom);
+        $xpath = new \DOMXPath($dom);
         // Some more counters
         $query = '/forum/discussions/discussion/posts/post';
         $posts = $xpath->query($query);
@@ -334,83 +362,83 @@ class backup_structure_testcase extends advanced_testcase {
                     $ratarr[$node->nodeName] = $node->nodeValue;
                 }
             }
-            $this->assertEquals($ratarr['userid'], $DB->get_field('rating', 'userid', array('id' => $ratarr['id'])));
-            $this->assertEquals($ratarr['itemid'], $DB->get_field('rating', 'itemid', array('id' => $ratarr['id'])));
-            $this->assertEquals($ratarr['post_rating'], $DB->get_field('rating', 'rating', array('id' => $ratarr['id'])));
+            $this->assertEquals($DB->get_field('rating', 'userid', array('id' => $ratarr['id'])), $ratarr['userid']);
+            $this->assertEquals($DB->get_field('rating', 'itemid', array('id' => $ratarr['id'])), $ratarr['itemid']);
+            $this->assertEquals($DB->get_field('rating', 'rating', array('id' => $ratarr['id'])), $ratarr['post_rating']);
         }
 
         // Check forum has "blockeperiod" with value 0 (was declared by object instead of name)
         $query = '/forum[blockperiod="0"]';
         $result = $xpath->query($query);
-        $this->assertEquals($result->length, 1);
+        $this->assertEquals(1, $result->length);
 
         // Check forum is missing "completiondiscussions" (as we are using mock_skip_final_element)
         $query = '/forum/completiondiscussions';
         $result = $xpath->query($query);
-        $this->assertEquals($result->length, 0);
+        $this->assertEquals(0, $result->length);
 
         // Check forum has "completionreplies" with value "original was 0, now changed" (because of mock_modify_final_element)
         $query = '/forum[completionreplies="original was 0, now changed"]';
         $result = $xpath->query($query);
-        $this->assertEquals($result->length, 1);
+        $this->assertEquals(1, $result->length);
 
         // Check forum has "completionposts" with value "intercepted!" (because of mock_final_element_interceptor)
         $query = '/forum[completionposts="intercepted!"]';
         $result = $xpath->query($query);
-        $this->assertEquals($result->length, 1);
+        $this->assertEquals(1, $result->length);
 
         // Check there isn't any alternative2 tag, as far as it hasn't source defined
         $query = '//alternative2';
         $result = $xpath->query($query);
-        $this->assertEquals($result->length, 0);
+        $this->assertEquals(0, $result->length);
 
         // Check there are 4 "field1" elements
         $query = '/forum/discussions/discussion/posts/post//field1';
         $result = $xpath->query($query);
-        $this->assertEquals($result->length, 4);
+        $this->assertEquals(4, $result->length);
 
         // Check first post has one name element with value "alternative1"
-        $query = '/forum/discussions/discussion/posts/post[@id="1"][name="alternative1"]';
+        $query = '/forum/discussions/discussion/posts/post[@id="'.$this->postid1.'"][name="alternative1"]';
         $result = $xpath->query($query);
-        $this->assertEquals($result->length, 1);
+        $this->assertEquals(1, $result->length);
 
         // Check there are two "dupetest1" elements
         $query = '/forum/discussions/discussion/posts/post//dupetest1';
         $result = $xpath->query($query);
-        $this->assertEquals($result->length, 2);
+        $this->assertEquals(2, $result->length);
 
         // Check second post has one name element with value "dupetest2"
-        $query = '/forum/discussions/discussion/posts/post[@id="2"]/dupetest2';
+        $query = '/forum/discussions/discussion/posts/post[@id="'.$this->postid2.'"]/dupetest2';
         $result = $xpath->query($query);
-        $this->assertEquals($result->length, 1);
+        $this->assertEquals(1, $result->length);
 
         // Check element "dupetest2" of second post has one field1 element with value "2"
-        $query = '/forum/discussions/discussion/posts/post[@id="2"]/dupetest2[field1="2"]';
+        $query = '/forum/discussions/discussion/posts/post[@id="'.$this->postid2.'"]/dupetest2[field1="2"]';
         $result = $xpath->query($query);
-        $this->assertEquals($result->length, 1);
+        $this->assertEquals(1, $result->length);
 
         // Check forth post has no name element
-        $query = '/forum/discussions/discussion/posts/post[@id="4"]/name';
+        $query = '/forum/discussions/discussion/posts/post[@id="'.$this->postid4.'"]/name';
         $result = $xpath->query($query);
-        $this->assertEquals($result->length, 0);
+        $this->assertEquals(0, $result->length);
 
         // Check 1st, 2nd and 3rd posts have no forumtype element
-        $query = '/forum/discussions/discussion/posts/post[@id="1"]/forumtype';
+        $query = '/forum/discussions/discussion/posts/post[@id="'.$this->postid1.'"]/forumtype';
         $result = $xpath->query($query);
-        $this->assertEquals($result->length, 0);
-        $query = '/forum/discussions/discussion/posts/post[@id="2"]/forumtype';
+        $this->assertEquals(0, $result->length);
+        $query = '/forum/discussions/discussion/posts/post[@id="'.$this->postid2.'"]/forumtype';
         $result = $xpath->query($query);
-        $this->assertEquals($result->length, 0);
-        $query = '/forum/discussions/discussion/posts/post[@id="3"]/forumtype';
+        $this->assertEquals(0, $result->length);
+        $query = '/forum/discussions/discussion/posts/post[@id="'.$this->postid3.'"]/forumtype';
         $result = $xpath->query($query);
-        $this->assertEquals($result->length, 0);
+        $this->assertEquals(0, $result->length);
 
         // Check 4th post has one forumtype element with value "general"
         // (because it doesn't matches alternatives 1, 2, 3, then alternative 4,
         // the one without conditions is being applied)
-        $query = '/forum/discussions/discussion/posts/post[@id="4"][forumtype="general"]';
+        $query = '/forum/discussions/discussion/posts/post[@id="'.$this->postid4.'"][forumtype="general"]';
         $result = $xpath->query($query);
-        $this->assertEquals($result->length, 1);
+        $this->assertEquals(1, $result->length);
 
         // Check annotations information against DB
         // Count records in original tables
@@ -454,7 +482,7 @@ class backup_structure_testcase extends advanced_testcase {
         try {
             $processor->set_var('onenewvariable', 999);
             $this->assertTrue(false, 'backup_processor_exception expected');
-        } catch (exception $e) {
+        } catch (\Exception $e) {
             $this->assertTrue($e instanceof backup_processor_exception);
             $this->assertEquals($e->errorcode, 'processorvariablealreadyset');
             $this->assertEquals($e->a, 'onenewvariable');
@@ -464,7 +492,7 @@ class backup_structure_testcase extends advanced_testcase {
         try {
             $var = $processor->get_var('nonexistingvar');
             $this->assertTrue(false, 'backup_processor_exception expected');
-        } catch (exception $e) {
+        } catch (\Exception $e) {
             $this->assertTrue($e instanceof backup_processor_exception);
             $this->assertEquals($e->errorcode, 'processorvariablenotfound');
             $this->assertEquals($e->a, 'nonexistingvar');
@@ -476,7 +504,7 @@ class backup_structure_testcase extends advanced_testcase {
             $ne->set_source_table('forum', array('id' => backup::VAR_PARENTID));
             $ne->process($processor);
             $this->assertTrue(false, 'base_element_struct_exception expected');
-        } catch (exception $e) {
+        } catch (\Exception $e) {
             $this->assertTrue($e instanceof base_element_struct_exception);
             $this->assertEquals($e->errorcode, 'cannotfindparentidforelement');
         }
@@ -484,25 +512,25 @@ class backup_structure_testcase extends advanced_testcase {
         // Try to process one nested/final/attribute elements without processor
         $ne = new backup_nested_element('test', 'one', 'two', 'three');
         try {
-            $ne->process(new stdclass());
+            $ne->process(new \stdClass());
             $this->assertTrue(false, 'base_element_struct_exception expected');
-        } catch (exception $e) {
+        } catch (\Exception $e) {
             $this->assertTrue($e instanceof base_element_struct_exception);
             $this->assertEquals($e->errorcode, 'incorrect_processor');
         }
         $fe = new backup_final_element('test');
         try {
-            $fe->process(new stdclass());
+            $fe->process(new \stdClass());
             $this->assertTrue(false, 'base_element_struct_exception expected');
-        } catch (exception $e) {
+        } catch (\Exception $e) {
             $this->assertTrue($e instanceof base_element_struct_exception);
             $this->assertEquals($e->errorcode, 'incorrect_processor');
         }
         $at = new backup_attribute('test');
         try {
-            $at->process(new stdclass());
+            $at->process(new \stdClass());
             $this->assertTrue(false, 'base_element_struct_exception expected');
-        } catch (exception $e) {
+        } catch (\Exception $e) {
             $this->assertTrue($e instanceof base_element_struct_exception);
             $this->assertEquals($e->errorcode, 'incorrect_processor');
         }
@@ -512,7 +540,7 @@ class backup_structure_testcase extends advanced_testcase {
         try {
             $ne->set_source_alias('last', 'nonexisting');
             $this->assertTrue(false, 'base_element_struct_exception expected');
-        } catch (exception $e) {
+        } catch (\Exception $e) {
             $this->assertTrue($e instanceof base_element_struct_exception);
             $this->assertEquals($e->errorcode, 'incorrectaliasfinalnamenotfound');
             $this->assertEquals($e->a, 'nonexisting');
@@ -523,7 +551,7 @@ class backup_structure_testcase extends advanced_testcase {
         try {
             $ne->set_source_table('forum', array('/test/subtest'));
             $this->assertTrue(false, 'base_element_struct_exception expected');
-        } catch (exception $e) {
+        } catch (\Exception $e) {
             $this->assertTrue($e instanceof base_element_struct_exception);
             $this->assertEquals($e->errorcode, 'baseelementincorrectfinalorattribute');
             $this->assertEquals($e->a, 'subtest');
@@ -531,7 +559,7 @@ class backup_structure_testcase extends advanced_testcase {
         try {
             $ne->set_source_table('forum', array('/wrongtest'));
             $this->assertTrue(false, 'base_element_struct_exception expected');
-        } catch (exception $e) {
+        } catch (\Exception $e) {
             $this->assertTrue($e instanceof base_element_struct_exception);
             $this->assertEquals($e->errorcode, 'baseelementincorrectgrandparent');
             $this->assertEquals($e->a, 'wrongtest');
@@ -539,7 +567,7 @@ class backup_structure_testcase extends advanced_testcase {
         try {
             $ne->set_source_table('forum', array('../nonexisting'));
             $this->assertTrue(false, 'base_element_struct_exception expected');
-        } catch (exception $e) {
+        } catch (\Exception $e) {
             $this->assertTrue($e instanceof base_element_struct_exception);
             $this->assertEquals($e->errorcode, 'baseelementincorrectparent');
             $this->assertEquals($e->a, '..');
@@ -552,7 +580,7 @@ class backup_structure_testcase extends advanced_testcase {
         try {
             $ne->annotate_files('test', 'filearea', null); // Try to add annotations twice
             $this->assertTrue(false, 'base_element_struct_exception expected');
-        } catch (exception $e) {
+        } catch (\Exception $e) {
             $this->assertTrue($e instanceof base_element_struct_exception);
             $this->assertEquals($e->errorcode, 'annotate_files_duplicate_annotation');
             $this->assertEquals($e->a, 'test/filearea/');
@@ -562,7 +590,7 @@ class backup_structure_testcase extends advanced_testcase {
         try {
             $ne->annotate_files('test', 'filearea', 'four'); // Incorrect element
             $this->assertTrue(false, 'base_element_struct_exception expected');
-        } catch (exception $e) {
+        } catch (\Exception $e) {
             $this->assertTrue($e instanceof base_element_struct_exception);
             $this->assertEquals($e->errorcode, 'baseelementincorrectfinalorattribute');
             $this->assertEquals($e->a, 'four');
@@ -573,7 +601,7 @@ class backup_structure_testcase extends advanced_testcase {
         try {
             $bog->add_child(new backup_nested_element('test2'));
             $this->assertTrue(false, 'base_optigroup_exception expected');
-        } catch (exception $e) {
+        } catch (\Exception $e) {
             $this->assertTrue($e instanceof base_optigroup_exception);
             $this->assertEquals($e->errorcode, 'optigroup_element_incorrect');
             $this->assertEquals($e->a, 'backup_nested_element');
@@ -583,16 +611,16 @@ class backup_structure_testcase extends advanced_testcase {
         try {
             $bog->add_child('test2');
             $this->assertTrue(false, 'base_optigroup_exception expected');
-        } catch (exception $e) {
+        } catch (\Exception $e) {
             $this->assertTrue($e instanceof base_optigroup_exception);
             $this->assertEquals($e->errorcode, 'optigroup_element_incorrect');
             $this->assertEquals($e->a, 'non object');
         }
 
         try {
-            $bog = new backup_optigroup('test', new stdclass());
+            $bog = new backup_optigroup('test', new \stdClass());
             $this->assertTrue(false, 'base_optigroup_exception expected');
-        } catch (exception $e) {
+        } catch (\Exception $e) {
             $this->assertTrue($e instanceof base_optigroup_exception);
             $this->assertEquals($e->errorcode, 'optigroup_elements_incorrect');
         }
@@ -600,9 +628,9 @@ class backup_structure_testcase extends advanced_testcase {
         // Try a wrong processor with backup_optigroup
         $bog = new backup_optigroup('test');
         try {
-            $bog->process(new stdclass());
+            $bog->process(new \stdClass());
             $this->assertTrue(false, 'base_element_struct_exception expected');
-        } catch (exception $e) {
+        } catch (\Exception $e) {
             $this->assertTrue($e instanceof base_element_struct_exception);
             $this->assertEquals($e->errorcode, 'incorrect_processor');
         }
@@ -620,7 +648,7 @@ class backup_structure_testcase extends advanced_testcase {
         try {
             $boge2->add_child($ne2);
             $this->assertTrue(false, 'base_optigroup_exception expected');
-        } catch (exception $e) {
+        } catch (\Exception $e) {
             $this->assertTrue($e instanceof base_optigroup_exception);
             $this->assertEquals($e->errorcode, 'multiple_optigroup_duplicate_element');
             $this->assertEquals($e->a, 'ne1');
@@ -637,7 +665,7 @@ class backup_structure_testcase extends advanced_testcase {
         try {
             $bog->add_child($boge2);
             $this->assertTrue(false, 'base_element_struct_exception expected');
-        } catch (exception $e) {
+        } catch (\Exception $e) {
             $this->assertTrue($e instanceof base_element_struct_exception);
             $this->assertEquals($e->errorcode, 'baseelementexisting');
             $this->assertEquals($e->a, 'ne1');
