@@ -18,8 +18,9 @@ declare(strict_types=1);
 
 namespace core_course\reportbuilder\datasource;
 
+use core\context\course;
 use core_reportbuilder_generator;
-use core_reportbuilder\local\filters\{boolean_select, date, select, tags, text};
+use core_reportbuilder\local\filters\{boolean_select, category, date, select, tags, text};
 use core_reportbuilder\tests\core_reportbuilder_testcase;
 
 /**
@@ -40,7 +41,13 @@ final class courses_test extends core_reportbuilder_testcase {
 
         // Test subject.
         $category = $this->getDataGenerator()->create_category(['name' => 'My cats']);
-        $course = $this->getDataGenerator()->create_course([
+        $courseone = $this->getDataGenerator()->create_course([
+            'category' => $category->id,
+            'fullname' => 'Feline fine',
+            'shortname' => 'C102',
+            'idnumber' => 'CAT102'
+        ]);
+        $coursetwo = $this->getDataGenerator()->create_course([
             'category' => $category->id,
             'fullname' => 'All about cats',
             'shortname' => 'C101',
@@ -52,16 +59,12 @@ final class courses_test extends core_reportbuilder_testcase {
         $report = $generator->create_report(['name' => 'Courses', 'source' => courses::class, 'default' => 1]);
 
         $content = $this->get_custom_report_content($report->get('id'));
-        $this->assertCount(1, $content);
-
-        $contentrow = array_values($content[0]);
 
+        // Default columns are category, shortname, fullname, idnumber. Sorted by category, shortname, fullname.
         $this->assertEquals([
-            $category->get_formatted_name(),
-            $course->shortname,
-            $course->fullname,
-            $course->idnumber,
-        ], $contentrow);
+            [$category->name, $coursetwo->shortname, $coursetwo->fullname, $coursetwo->idnumber],
+            [$category->name, $courseone->shortname, $courseone->fullname, $courseone->idnumber],
+        ], array_map('array_values', $content));
     }
 
     /**
@@ -82,6 +85,16 @@ final class courses_test extends core_reportbuilder_testcase {
             'tags' => ['Horses'],
         ]);
 
+        // Add a course image.
+        get_file_storage()->create_file_from_string([
+            'contextid' => course::instance($course->id)->id,
+            'component' => 'course',
+            'filearea' => 'overviewfiles',
+            'itemid' => 0,
+            'filepath' => '/',
+            'filename' => 'HelloWorld.jpg',
+        ], 'HelloWorld');
+
         /** @var core_reportbuilder_generator $generator */
         $generator = $this->getDataGenerator()->get_plugin_generator('core_reportbuilder');
         $report = $generator->create_report(['name' => 'Courses', 'source' => courses::class, 'default' => 0]);
@@ -113,6 +126,9 @@ final class courses_test extends core_reportbuilder_testcase {
         $generator->create_column(['reportid' => $report->get('id'), 'uniqueidentifier' => 'tag:name']);
         $generator->create_column(['reportid' => $report->get('id'), 'uniqueidentifier' => 'tag:namewithlink']);
 
+        // File entity.
+        $generator->create_column(['reportid' => $report->get('id'), 'uniqueidentifier' => 'file:name']);
+
         $content = $this->get_custom_report_content($report->get('id'));
         $this->assertCount(1, $content);
 
@@ -144,6 +160,9 @@ final class courses_test extends core_reportbuilder_testcase {
         // Tags.
         $this->assertEquals('Horses', $courserow[19]);
         $this->assertStringContainsString('Horses', $courserow[20]);
+
+        // File.
+        $this->assertEquals('HelloWorld.jpg', $courserow[21]);
     }
 
     /**
@@ -193,6 +212,11 @@ final class courses_test extends core_reportbuilder_testcase {
         return [
             // Category.
             'Filter category' => ['course_category:name', [
+                'course_category:name_operator' => category::NOT_EQUAL_TO,
+                'course_category:name_value' => -1,
+            ], true],
+            'Filter category (no match)' => ['course_category:name', [
+                'course_category:name_operator' => category::EQUAL_TO,
                 'course_category:name_value' => -1,
             ], false],
             'Filter category name' => ['course_category:text', [
@@ -335,6 +359,11 @@ final class courses_test extends core_reportbuilder_testcase {
             'Filter tag name not empty' => ['tag:name', [
                 'tag:name_operator' => tags::NOT_EMPTY,
             ], true],
+
+            // File.
+            'Filter file name empty' => ['file:name', [
+                'file:name_operator' => text::IS_EMPTY,
+            ], true],
         ];
     }
 
