@@ -123,6 +123,18 @@ abstract class field_controller {
         return $fieldcontroller;
     }
 
+    /**
+     * Perform pre-processing of field values, for example those that originate from an external source (e.g. upload course tool)
+     *
+     * Override in plugin classes as necessary
+     *
+     * @param string $value
+     * @return mixed
+     */
+    public function parse_value(string $value) {
+        return $value;
+    }
+
     /**
      * Validate the data on the field configuration form
      *
@@ -243,11 +255,15 @@ abstract class field_controller {
     /**
      * Returns the field name formatted according to configuration context.
      *
+     * @param bool $escape
      * @return string
      */
-    public function get_formatted_name() : string {
+    public function get_formatted_name(bool $escape = true): string {
         $context = $this->get_handler()->get_configuration_context();
-        return format_string($this->get('name'), true, ['context' => $context]);
+        return format_string($this->get('name'), true, [
+            'context' => $context,
+            'escape' => $escape,
+        ]);
     }
 
     /**
