@@ -24,6 +24,7 @@
  */
 
 require_once(__DIR__.'/../../testing/classes/util.php');
+require_once(__DIR__ . "/coverage_info.php");
 
 /**
  * Collection of utility methods.
@@ -62,7 +63,7 @@ class phpunit_util extends testing_util {
     /**
      * @var array Files to skip when dropping dataroot folder
      */
-    protected static $datarootskipondrop = array('.', '..', 'lock', 'webrunner.xml');
+    protected static $datarootskipondrop = array('.', '..', 'lock');
 
     /**
      * Load global $CFG;
@@ -101,7 +102,10 @@ class phpunit_util extends testing_util {
      * @return void
      */
     public static function reset_all_data($detectchanges = false) {
-        global $DB, $CFG, $USER, $SITE, $COURSE, $PAGE, $OUTPUT, $SESSION, $FULLME;
+        global $DB, $CFG, $USER, $SITE, $COURSE, $PAGE, $OUTPUT, $SESSION, $FULLME, $FILTERLIB_PRIVATE;
+
+        // Stop all hook redirections.
+        \core\hook\manager::get_instance()->phpunit_stop_redirections();
 
         // Stop any message redirection.
         self::stop_message_redirection();
@@ -201,6 +205,10 @@ class phpunit_util extends testing_util {
         $FULLME = null;
         $ME = null;
         $SCRIPT = null;
+        $FILTERLIB_PRIVATE = null;
+        if (!empty($SESSION->notifications)) {
+            $SESSION->notifications = [];
+        }
 
         // Empty sessison and set fresh new not-logged-in user.
         \core\session\manager::init_empty_session();
@@ -208,15 +216,18 @@ class phpunit_util extends testing_util {
         // reset all static caches
         \core\event\manager::phpunit_reset();
         accesslib_clear_all_caches(true);
+        accesslib_reset_role_cache();
         get_string_manager()->reset_caches(true);
         reset_text_filters_cache(true);
-        events_get_handlers('reset');
-        core_text::reset_caches();
         get_message_processors(false, true, true);
         filter_manager::reset_caches();
         core_filetypes::reset_caches();
         \core_search\manager::clear_static();
         core_user::reset_caches();
+        \core\output\icon_system::reset_caches();
+        if (class_exists('core_media_manager', false)) {
+            core_media_manager::reset_caches();
+        }
 
         // Reset static unit test options.
         if (class_exists('\availability_date\condition', false)) {
@@ -226,13 +237,15 @@ class phpunit_util extends testing_util {
         // Reset internal users.
         core_user::reset_internal_users();
 
+        // Clear static caches in calendar container.
+        if (class_exists('\core_calendar\local\event\container', false)) {
+            core_calendar\local\event\container::reset_caches();
+        }
+
         //TODO MDL-25290: add more resets here and probably refactor them to new core function
 
         // Reset course and module caches.
-        if (class_exists('format_base')) {
-            // If file containing class is not loaded, there is no cache there anyway.
-            format_base::reset_course_cache(0);
-        }
+        core_courseformat\base::reset_course_cache(0);
         get_fast_modinfo(0, 0, true);
 
         // Reset other singletons.
@@ -242,6 +255,21 @@ class phpunit_util extends testing_util {
         if (class_exists('\core\update\checker')) {
             \core\update\checker::reset_caches(true);
         }
+        if (class_exists('\core_course\customfield\course_handler')) {
+            \core_course\customfield\course_handler::reset_caches();
+        }
+        if (class_exists('\core_reportbuilder\manager')) {
+            \core_reportbuilder\manager::reset_caches();
+        }
+        if (class_exists('\core_cohort\customfield\cohort_handler')) {
+            \core_cohort\customfield\cohort_handler::reset_caches();
+        }
+        if (class_exists('\core_group\customfield\group_handler')) {
+            \core_group\customfield\group_handler::reset_caches();
+        }
+        if (class_exists('\core_group\customfield\grouping_handler')) {
+            \core_group\customfield\grouping_handler::reset_caches();
+        }
 
         // Clear static cache within restore.
         if (class_exists('restore_section_structure_step')) {
@@ -269,6 +297,9 @@ class phpunit_util extends testing_util {
         // Reset the log manager cache.
         get_log_manager(true);
 
+        // Reset user agent.
+        core_useragent::instance(true, null);
+
         // verify db writes just in case something goes wrong in reset
         if (self::$lastdbwrites != $DB->perf_get_writes()) {
             error_log('Unexpected DB writes in phpunit_util::reset_all_data()');
@@ -289,7 +320,13 @@ class phpunit_util extends testing_util {
     public static function reset_database() {
         global $DB;
 
-        if (!is_null(self::$lastdbwrites) and self::$lastdbwrites == $DB->perf_get_writes()) {
+        if (defined('PHPUNIT_ISOLATED_TEST') && PHPUNIT_ISOLATED_TEST && self::$lastdbwrites === null) {
+            // This is an isolated test and the lastdbwrites has not yet been initialised.
+            // Isolated test runs are reset by the test runner before the run starts.
+            self::$lastdbwrites = $DB->perf_get_writes();
+        }
+
+        if (!is_null(self::$lastdbwrites) && self::$lastdbwrites == $DB->perf_get_writes()) {
             return false;
         }
 
@@ -409,10 +446,12 @@ class phpunit_util extends testing_util {
 
         self::reset_dataroot();
         testing_initdataroot($CFG->dataroot, 'phpunit');
-        self::drop_dataroot();
 
-        // drop all tables
+        // Drop all tables.
         self::drop_database($displayprogress);
+
+        // Drop dataroot.
+        self::drop_dataroot();
     }
 
     /**
@@ -452,6 +491,16 @@ class phpunit_util extends testing_util {
         // Disable all logging for performance and sanity reasons.
         set_config('enabled_stores', '', 'tool_log');
 
+        // Remove any default blocked hosts and port restrictions, to avoid blocking tests (eg those using local files).
+        set_config('curlsecurityblockedhosts', '');
+        set_config('curlsecurityallowedport', '');
+
+        // Execute all the adhoc tasks.
+        while ($task = \core\task\manager::get_next_adhoc_task(time())) {
+            $task->execute();
+            \core\task\manager::adhoc_task_complete($task);
+        }
+
         // We need to keep the installed dataroot filedir files.
         // So each time we reset the dataroot before running a test, the default files are still installed.
         self::save_original_data_files();
@@ -464,51 +513,87 @@ class phpunit_util extends testing_util {
     }
 
     /**
-     * Builds dirroot/phpunit.xml and dataroot/phpunit/webrunner.xml files using defaults from /phpunit.xml.dist
+     * Builds dirroot/phpunit.xml file using defaults from /phpunit.xml.dist
      * @static
      * @return bool true means main config file created, false means only dataroot file created
      */
     public static function build_config_file() {
         global $CFG;
 
-        $template = '
-        <testsuite name="@component@_testsuite">
-            <directory suffix="_test.php">@dir@</directory>
-        </testsuite>';
+        $template = <<<EOF
+            <testsuite name="@component@_testsuite">
+              <directory suffix="_test.php">@dir@</directory>
+              <exclude>@dir@/classes</exclude>
+              <exclude>@dir@/fixtures</exclude>
+            </testsuite>
+
+        EOF;
         $data = file_get_contents("$CFG->dirroot/phpunit.xml.dist");
 
         $suites = '';
+        $includelists = [];
+        $excludelists = [];
+
+        $subsystems = core_component::get_core_subsystems();
+        $subsystems['core'] = $CFG->dirroot . '/lib';
+        foreach ($subsystems as $subsystem => $fulldir) {
+            if (empty($fulldir)) {
+                continue;
+            }
+            if (!file_exists("{$fulldir}/tests/")) {
+                // There are no tests - skip this directory.
+                continue;
+            }
+
+            $dir = substr($fulldir, strlen($CFG->dirroot) + 1);
+            if ($coverageinfo = self::get_coverage_info($fulldir)) {
+                $includelists = array_merge($includelists, $coverageinfo->get_includelists($dir));
+                $excludelists = array_merge($excludelists, $coverageinfo->get_excludelists($dir));
+            }
+        }
 
         $plugintypes = core_component::get_plugin_types();
         ksort($plugintypes);
-        foreach ($plugintypes as $type=>$unused) {
+        foreach (array_keys($plugintypes) as $type) {
             $plugs = core_component::get_plugin_list($type);
             ksort($plugs);
-            foreach ($plugs as $plug=>$fullplug) {
-                if (!file_exists("$fullplug/tests/")) {
+            foreach ($plugs as $plug => $plugindir) {
+                if (!file_exists("{$plugindir}/tests/")) {
+                    // There are no tests - skip this directory.
                     continue;
                 }
-                $dir = substr($fullplug, strlen($CFG->dirroot)+1);
-                $dir .= '/tests';
-                $component = $type.'_'.$plug;
+
+                $dir = substr($plugindir, strlen($CFG->dirroot) + 1);
+                $testdir = "{$dir}/tests";
+                $component = "{$type}_{$plug}";
 
                 $suite = str_replace('@component@', $component, $template);
-                $suite = str_replace('@dir@', $dir, $suite);
+                $suite = str_replace('@dir@', $testdir, $suite);
 
                 $suites .= $suite;
+
+                if ($coverageinfo = self::get_coverage_info($plugindir)) {
+
+                    $includelists = array_merge($includelists, $coverageinfo->get_includelists($dir));
+                    $excludelists = array_merge($excludelists, $coverageinfo->get_excludelists($dir));
+                }
             }
         }
+
         // Start a sequence between 100000 and 199000 to ensure each call to init produces
         // different ids in the database.  This reduces the risk that hard coded values will
         // end up being placed in phpunit or behat test code.
         $sequencestart = 100000 + mt_rand(0, 99) * 1000;
 
-        $data = preg_replace('|<!--@plugin_suites_start@-->.*<!--@plugin_suites_end@-->|s', $suites, $data, 1);
+        $data = preg_replace('| *<!--@plugin_suites_start@-->.*<!--@plugin_suites_end@-->|s', trim($suites, "\n"), $data, 1);
         $data = str_replace(
             '<const name="PHPUNIT_SEQUENCE_START" value=""/>',
             '<const name="PHPUNIT_SEQUENCE_START" value="' . $sequencestart . '"/>',
             $data);
 
+        $coverages = self::get_coverage_config($includelists, $excludelists);
+        $data = preg_replace('| *<!--@coveragelist@-->|s', trim($coverages, "\n"), $data);
+
         $result = false;
         if (is_writable($CFG->dirroot)) {
             if ($result = file_put_contents("$CFG->dirroot/phpunit.xml", $data)) {
@@ -516,14 +601,6 @@ class phpunit_util extends testing_util {
             }
         }
 
-        // relink - it seems that xml:base does not work in phpunit xml files, remove this nasty hack if you find a way to set xml base for relative refs
-        $data = str_replace('lib/phpunit/', $CFG->dirroot.DIRECTORY_SEPARATOR.'lib'.DIRECTORY_SEPARATOR.'phpunit'.DIRECTORY_SEPARATOR, $data);
-        $data = preg_replace('|<directory suffix="_test.php">([^<]+)</directory>|',
-            '<directory suffix="_test.php">'.$CFG->dirroot.(DIRECTORY_SEPARATOR === '\\' ? '\\\\' : DIRECTORY_SEPARATOR).'$1</directory>',
-            $data);
-        file_put_contents("$CFG->dataroot/phpunit/webrunner.xml", $data);
-        testing_fix_file_permissions("$CFG->dataroot/phpunit/webrunner.xml");
-
         return (bool)$result;
     }
 
@@ -536,12 +613,23 @@ class phpunit_util extends testing_util {
     public static function build_component_config_files() {
         global $CFG;
 
-        $template = '
-        <testsuites>
-            <testsuite name="@component@_testsuite">
-                <directory suffix="_test.php">.</directory>
-            </testsuite>
-        </testsuites>';
+        $template = <<<EOT
+            <testsuites>
+              <testsuite name="@component@_testsuite">
+                <directory suffix="_test.php">tests</directory>
+                <exclude>tests/classes</exclude>
+                <exclude>tests/fixtures</exclude>
+              </testsuite>
+            </testsuites>
+          EOT;
+        $coveragedefault = <<<EOT
+            <include>
+              <directory suffix=".php">.</directory>
+            </include>
+            <exclude>
+              <directory suffix="_test.php">tests</directory>
+            </exclude>
+        EOT;
 
         // Start a sequence between 100000 and 199000 to ensure each call to init produces
         // different ids in the database.  This reduces the risk that hard coded values will
@@ -550,7 +638,7 @@ class phpunit_util extends testing_util {
 
         // Use the upstream file as source for the distributed configurations
         $ftemplate = file_get_contents("$CFG->dirroot/phpunit.xml.dist");
-        $ftemplate = preg_replace('|<!--All core suites.*</testsuites>|s', '<!--@component_suite@-->', $ftemplate);
+        $ftemplate = preg_replace('| *<!--All core suites.*</testsuites>|s', '<!--@component_suite@-->', $ftemplate);
 
         // Gets all the components with tests
         $components = tests_finder::get_components_with_tests('phpunit');
@@ -561,8 +649,17 @@ class phpunit_util extends testing_util {
             $ctemplate = $template;
             $ctemplate = str_replace('@component@', $cname, $ctemplate);
 
-            // Apply it to the file template
             $fcontents = str_replace('<!--@component_suite@-->', $ctemplate, $ftemplate);
+
+            // Check for coverage configurations.
+            if ($coverageinfo = self::get_coverage_info($cpath)) {
+                $coverages = self::get_coverage_config($coverageinfo->get_includelists(''), $coverageinfo->get_excludelists(''));
+            } else {
+                $coverages = $coveragedefault;
+            }
+            $fcontents = preg_replace('| *<!--@coveragelist@-->|s', trim($coverages, "\n"), $fcontents);
+
+            // Apply it to the file template.
             $fcontents = str_replace(
                 '<const name="PHPUNIT_SEQUENCE_START" value=""/>',
                 '<const name="PHPUNIT_SEQUENCE_START" value="' . $sequencestart . '"/>',
@@ -597,9 +694,30 @@ class phpunit_util extends testing_util {
         // we need normal debugging outside of tests to find problems in our phpunit integration.
         $backtrace = debug_backtrace();
 
+        // Only for advanced_testcase, database_driver_testcase (and descendants). Others aren't
+        // able to manage the debugging sink, so any debugging has to be output normally and, hopefully,
+        // PHPUnit execution will catch that unexpected output properly.
+        $sinksupport = false;
         foreach ($backtrace as $bt) {
-            if (isset($bt['object']) and is_object($bt['object'])
-                    && $bt['object'] instanceof PHPUnit_Framework_TestCase) {
+            if (isset($bt['object']) && is_object($bt['object'])
+                && (
+                    $bt['object'] instanceof advanced_testcase ||
+                    $bt['object'] instanceof database_driver_testcase)
+            ) {
+                $sinksupport = true;
+                break;
+            }
+        }
+        if (!$sinksupport) {
+            return false;
+        }
+
+        // Verify that we are inside a PHPUnit test (little bit redundant, because
+        // we already have checked above that this is an advanced/database_driver
+        // testcase, but let's keep things double safe for now).
+        foreach ($backtrace as $bt) {
+            if (isset($bt['object']) && is_object($bt['object'])
+                    && $bt['object'] instanceof PHPUnit\Framework\TestCase) {
                 $debug = new stdClass();
                 $debug->message = $message;
                 $debug->level   = $level;
@@ -692,7 +810,7 @@ class phpunit_util extends testing_util {
     /**
      * To be called from messagelib.php only!
      *
-     * @param stdClass $message record from message_read table
+     * @param stdClass $message record from messages table
      * @return bool true means send message, false means message "sent" to sink.
      */
     public static function message_sent($message) {
@@ -743,7 +861,7 @@ class phpunit_util extends testing_util {
     /**
      * To be called from messagelib.php only!
      *
-     * @param stdClass $message record from message_read table
+     * @param stdClass $message record from messages table
      * @return bool true means send message, false means message "sent" to sink.
      */
     public static function phpmailer_sent($message) {
@@ -819,4 +937,114 @@ class phpunit_util extends testing_util {
             return 'en_AU.UTF-8';
         }
     }
+
+    /**
+     * Executes all adhoc tasks in the queue. Useful for testing asynchronous behaviour.
+     *
+     * @return void
+     */
+    public static function run_all_adhoc_tasks() {
+        $now = time();
+        while (($task = \core\task\manager::get_next_adhoc_task($now)) !== null) {
+            try {
+                $task->execute();
+                \core\task\manager::adhoc_task_complete($task);
+            } catch (Exception $e) {
+                \core\task\manager::adhoc_task_failed($task);
+            }
+        }
+    }
+
+    /**
+     * Helper function to call a protected/private method of an object using reflection.
+     *
+     * Example 1. Calling a protected object method:
+     *   $result = call_internal_method($myobject, 'method_name', [$param1, $param2], '\my\namespace\myobjectclassname');
+     *
+     * Example 2. Calling a protected static method:
+     *   $result = call_internal_method(null, 'method_name', [$param1, $param2], '\my\namespace\myclassname');
+     *
+     * @param object|null $object the object on which to call the method, or null if calling a static method.
+     * @param string $methodname the name of the protected/private method.
+     * @param array $params the array of function params to pass to the method.
+     * @param string $classname the fully namespaced name of the class the object was created from (base in the case of mocks),
+     *        or the name of the static class when calling a static method.
+     * @return mixed the respective return value of the method.
+     */
+    public static function call_internal_method($object, $methodname, array $params, $classname) {
+        $reflection = new \ReflectionClass($classname);
+        $method = $reflection->getMethod($methodname);
+        $method->setAccessible(true);
+        return $method->invokeArgs($object, $params);
+    }
+
+    /**
+     * Pad the supplied string with $level levels of indentation.
+     *
+     * @param   string  $string The string to pad
+     * @param   int     $level The number of levels of indentation to pad
+     * @return  string
+     */
+    protected static function pad(string $string, int $level) : string {
+        return str_repeat(" ", $level * 2) . "{$string}\n";
+    }
+
+    /**
+     * Get the coverage config for the supplied includelist and excludelist configuration.
+     *
+     * @param   string[] $includelists The list of files/folders in the includelist.
+     * @param   string[] $excludelists The list of files/folders in the excludelist.
+     * @return  string
+     */
+    protected static function get_coverage_config(array $includelists, array $excludelists) : string {
+        $coverages = '';
+        if (!empty($includelists)) {
+            $coverages .= self::pad("<include>", 2);
+            foreach ($includelists as $line) {
+                $coverages .= self::pad($line, 3);
+            }
+            $coverages .= self::pad("</include>", 2);
+            if (!empty($excludelists)) {
+                $coverages .= self::pad("<exclude>", 2);
+                foreach ($excludelists as $line) {
+                    $coverages .= self::pad($line, 3);
+                }
+                $coverages .= self::pad("</exclude>", 2);
+            }
+        }
+
+        return $coverages;
+    }
+
+    /**
+     * Get the phpunit_coverage_info for the specified plugin or subsystem directory.
+     *
+     * @param   string  $fulldir The directory to find the coverage info file in.
+     * @return  phpunit_coverage_info
+     */
+    protected static function get_coverage_info(string $fulldir): phpunit_coverage_info {
+        $coverageconfig = "{$fulldir}/tests/coverage.php";
+        if (file_exists($coverageconfig)) {
+            $coverageinfo = require($coverageconfig);
+            if (!$coverageinfo instanceof phpunit_coverage_info) {
+                throw new \coding_exception("{$coverageconfig} does not return a phpunit_coverage_info");
+            }
+
+            return $coverageinfo;
+        }
+
+        return new phpunit_coverage_info();;
+    }
+
+    /**
+     * Whether the current process is an isolated test process.
+     *
+     * @return bool
+     */
+    public static function is_in_isolated_process(): bool {
+        // Note: There is no function to call, or much to go by in order to tell whether we are in an isolated process
+        // during Bootstrap, when this function is called.
+        // We can do so by testing the existence of the wrapper function, but there is nothing set until that point.
+        return function_exists('__phpunit_run_isolated_test');
+    }
 }
