@@ -24,7 +24,7 @@ namespace core;
  * @copyright 2012 The Open University
  * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
-class setuplib_test extends \advanced_testcase {
+final class setuplib_test extends \advanced_testcase {
 
     /**
      * Test get_docs_url_standard in the normal case when we should link to Moodle docs.
@@ -426,7 +426,7 @@ class setuplib_test extends \advanced_testcase {
      *
      * @return array An array of arrays contain test data
      */
-    public function data_for_test_get_real_size() {
+    public static function data_for_test_get_real_size(): array {
         return array(
             array('8KB',    8192),
             array('8Kb',    8192),
@@ -520,4 +520,26 @@ class setuplib_test extends \advanced_testcase {
         $uuid = \core\uuid::generate();
         $this->assertTrue(self::is_valid_uuid_v4($uuid), "Invalid v4 UUID: '$uuid'");
     }
+
+    /**
+     * Test require_phpunit_isolation in a test which is not isolated.
+     *
+     * @covers ::require_phpunit_isolation
+     */
+    public function test_require_phpunit_isolation(): void {
+        // A unit test which is not isolated will throw a coding_exception when the function is called.
+        $this->expectException('coding_exception');
+        require_phpunit_isolation();
+    }
+
+    /**
+     * Test require_phpunit_isolation in a test which is isolated.
+     *
+     * @covers ::require_phpunit_isolation
+     * @runInSeparateProcess
+     */
+    public function test_require_phpunit_isolation_isolated(): void {
+        $this->expectNotToPerformAssertions();
+        require_phpunit_isolation();
+    }
 }
