@@ -33,7 +33,7 @@ require_once($CFG->dirroot . '/calendar/tests/helpers.php');
 /**
  * Tests various classes and functions in upgradelib.php library.
  */
-class upgradelib_test extends advanced_testcase {
+final class upgradelib_test extends advanced_testcase {
 
     /**
      * Test the {@link upgrade_stale_php_files_present() function
@@ -744,7 +744,7 @@ class upgradelib_test extends advanced_testcase {
      *
      * @return array
      */
-    public function serialized_strings_dataprovider() {
+    public static function serialized_strings_dataprovider(): array {
         return [
             'A configuration that uses the old object' => [
                 'O:6:"object":3:{s:4:"text";s:32:"Nothing that anyone cares about.";s:5:"title";s:16:"Really old block";s:6:"format";s:1:"1";}',
@@ -894,214 +894,6 @@ class upgradelib_test extends advanced_testcase {
         $this->assertEquals($file, $newstoredfile[1]);
     }
 
-    /**
-     * Test that the previous records are updated according to the reworded actions.
-     * @return null
-     */
-    public function test_upgrade_rename_prediction_actions_useful_incorrectly_flagged() {
-        global $DB;
-
-        $this->resetAfterTest();
-        $this->setAdminUser();
-
-        $models = $DB->get_records('analytics_models');
-        $upcomingactivitiesdue = null;
-        $noteaching = null;
-        foreach ($models as $model) {
-            if ($model->target === '\\core_user\\analytics\\target\\upcoming_activities_due') {
-                $upcomingactivitiesdue = new \core_analytics\model($model);
-            }
-            if ($model->target === '\\core_course\\analytics\\target\\no_teaching') {
-                $noteaching = new \core_analytics\model($model);
-            }
-        }
-
-        // Upcoming activities due generating some insights.
-        $course1 = $this->getDataGenerator()->create_course();
-        $attrs = ['course' => $course1, 'duedate' => time() + WEEKSECS - DAYSECS];
-        $assign = $this->getDataGenerator()->get_plugin_generator('mod_assign')->create_instance($attrs);
-        $student = $this->getDataGenerator()->create_user();
-        $usercontext = \context_user::instance($student->id);
-        $this->getDataGenerator()->enrol_user($student->id, $course1->id, 'student');
-        $upcomingactivitiesdue->predict();
-        list($ignored, $predictions) = $upcomingactivitiesdue->get_predictions($usercontext, true);
-        $prediction = reset($predictions);
-
-        $predictionaction = (object)[
-            'predictionid' => $prediction->get_prediction_data()->id,
-            'userid' => 2,
-            'actionname' => 'fixed',
-            'timecreated' => time()
-        ];
-        $DB->insert_record('analytics_prediction_actions', $predictionaction);
-        $predictionaction->actionname = 'notuseful';
-        $DB->insert_record('analytics_prediction_actions', $predictionaction);
-
-        upgrade_rename_prediction_actions_useful_incorrectly_flagged();
-
-        $this->assertEquals(0, $DB->count_records('analytics_prediction_actions',
-            ['actionname' => \core_analytics\prediction::ACTION_FIXED]));
-        $this->assertEquals(1, $DB->count_records('analytics_prediction_actions',
-            ['actionname' => \core_analytics\prediction::ACTION_USEFUL]));
-        $this->assertEquals(1, $DB->count_records('analytics_prediction_actions',
-            ['actionname' => \core_analytics\prediction::ACTION_NOT_USEFUL]));
-        $this->assertEquals(0, $DB->count_records('analytics_prediction_actions',
-            ['actionname' => \core_analytics\prediction::ACTION_INCORRECTLY_FLAGGED]));
-
-        // No teaching generating some insights.
-        $course2 = $this->getDataGenerator()->create_course(['startdate' => time() + (2 * DAYSECS)]);
-        $noteaching->predict();
-        list($ignored, $predictions) = $noteaching->get_predictions(\context_system::instance(), true);
-        $prediction = reset($predictions);
-
-        $predictionaction = (object)[
-            'predictionid' => $prediction->get_prediction_data()->id,
-            'userid' => 2,
-            'actionname' => 'notuseful',
-            'timecreated' => time()
-        ];
-        $DB->insert_record('analytics_prediction_actions', $predictionaction);
-        $predictionaction->actionname = 'fixed';
-        $DB->insert_record('analytics_prediction_actions', $predictionaction);
-
-        upgrade_rename_prediction_actions_useful_incorrectly_flagged();
-
-        $this->assertEquals(1, $DB->count_records('analytics_prediction_actions',
-            ['actionname' => \core_analytics\prediction::ACTION_FIXED]));
-        $this->assertEquals(1, $DB->count_records('analytics_prediction_actions',
-            ['actionname' => \core_analytics\prediction::ACTION_USEFUL]));
-        $this->assertEquals(1, $DB->count_records('analytics_prediction_actions',
-            ['actionname' => \core_analytics\prediction::ACTION_NOT_USEFUL]));
-        $this->assertEquals(1, $DB->count_records('analytics_prediction_actions',
-            ['actionname' => \core_analytics\prediction::ACTION_INCORRECTLY_FLAGGED]));
-
-        // We also check that there are no records incorrectly switched in upcomingactivitiesdue.
-        $upcomingactivitiesdue->clear();
-
-        $this->assertEquals(1, $DB->count_records('analytics_prediction_actions',
-            ['actionname' => \core_analytics\prediction::ACTION_FIXED]));
-        $this->assertEquals(0, $DB->count_records('analytics_prediction_actions',
-            ['actionname' => \core_analytics\prediction::ACTION_USEFUL]));
-        $this->assertEquals(0, $DB->count_records('analytics_prediction_actions',
-            ['actionname' => \core_analytics\prediction::ACTION_NOT_USEFUL]));
-        $this->assertEquals(1, $DB->count_records('analytics_prediction_actions',
-            ['actionname' => \core_analytics\prediction::ACTION_INCORRECTLY_FLAGGED]));
-
-        $upcomingactivitiesdue->predict();
-        list($ignored, $predictions) = $upcomingactivitiesdue->get_predictions($usercontext, true);
-        $prediction = reset($predictions);
-
-        $predictionaction = (object)[
-            'predictionid' => $prediction->get_prediction_data()->id,
-            'userid' => 2,
-            'actionname' => 'fixed',
-            'timecreated' => time()
-        ];
-        $DB->insert_record('analytics_prediction_actions', $predictionaction);
-        $predictionaction->actionname = 'notuseful';
-        $DB->insert_record('analytics_prediction_actions', $predictionaction);
-
-        upgrade_rename_prediction_actions_useful_incorrectly_flagged();
-
-        $this->assertEquals(1, $DB->count_records('analytics_prediction_actions',
-            ['actionname' => \core_analytics\prediction::ACTION_FIXED]));
-        $this->assertEquals(1, $DB->count_records('analytics_prediction_actions',
-            ['actionname' => \core_analytics\prediction::ACTION_USEFUL]));
-        $this->assertEquals(1, $DB->count_records('analytics_prediction_actions',
-            ['actionname' => \core_analytics\prediction::ACTION_NOT_USEFUL]));
-        $this->assertEquals(1, $DB->count_records('analytics_prediction_actions',
-            ['actionname' => \core_analytics\prediction::ACTION_INCORRECTLY_FLAGGED]));
-    }
-
-    /**
-     * Test the functionality of the {@link upgrade_convert_hub_config_site_param_names()} function.
-     */
-    public function test_upgrade_convert_hub_config_site_param_names() {
-
-        $config = (object) [
-            // This is how site settings related to registration at https://moodle.net are stored.
-            'site_name_httpsmoodlenet' => 'Foo Site',
-            'site_language_httpsmoodlenet' => 'en',
-            'site_emailalert_httpsmoodlenet' => 1,
-            // These are unexpected relics of a value as registered at the old http://hub.moodle.org site.
-            'site_name_httphubmoodleorg' => 'Bar Site',
-            'site_description_httphubmoodleorg' => 'Old description',
-            // This is the target value we are converting to - here it already somehow exists.
-            'site_emailalert' => 0,
-            // This is a setting not related to particular hub.
-            'custom' => 'Do not touch this',
-            // A setting defined for multiple alternative hubs.
-            'site_foo_httpfirsthuborg' => 'First',
-            'site_foo_httpanotherhubcom' => 'Another',
-            'site_foo_httpyetanotherhubcom' => 'Yet another',
-            // A setting defined for multiple alternative hubs and one referential one.
-            'site_bar_httpfirsthuborg' => 'First',
-            'site_bar_httpanotherhubcom' => 'Another',
-            'site_bar_httpsmoodlenet' => 'One hub to rule them all!',
-            'site_bar_httpyetanotherhubcom' => 'Yet another',
-        ];
-
-        $converted = upgrade_convert_hub_config_site_param_names($config, 'https://moodle.net');
-
-        // Values defined for the moodle.net take precedence over the ones defined for other hubs.
-        $this->assertSame($converted->site_name, 'Foo Site');
-        $this->assertSame($converted->site_bar, 'One hub to rule them all!');
-        $this->assertNull($converted->site_name_httpsmoodlenet);
-        $this->assertNull($converted->site_bar_httpfirsthuborg);
-        $this->assertNull($converted->site_bar_httpanotherhubcom);
-        $this->assertNull($converted->site_bar_httpyetanotherhubcom);
-        // Values defined for alternative hubs only do not have any guaranteed value. Just for convenience, we use the first one.
-        $this->assertSame($converted->site_foo, 'First');
-        $this->assertNull($converted->site_foo_httpfirsthuborg);
-        $this->assertNull($converted->site_foo_httpanotherhubcom);
-        $this->assertNull($converted->site_foo_httpyetanotherhubcom);
-        // Values that are already defined with the new name format are kept.
-        $this->assertSame($converted->site_emailalert, 0);
-        // Eventual custom values not following the expected hub-specific naming format, are kept.
-        $this->assertSame($converted->custom, 'Do not touch this');
-    }
-
-    /**
-     * Test the functionality of the {@link upgrade_analytics_fix_contextids_defaults} function.
-     */
-    public function test_upgrade_analytics_fix_contextids_defaults() {
-        global $DB, $USER;
-
-        $this->resetAfterTest();
-
-        $model = (object)[
-            'name' => 'asd',
-            'target' => 'ou',
-            'indicators' => '[]',
-            'version' => '1',
-            'timecreated' => time(),
-            'timemodified' => time(),
-            'usermodified' => $USER->id,
-            'contextids' => ''
-        ];
-        $DB->insert_record('analytics_models', $model);
-
-        $model->contextids = null;
-        $DB->insert_record('analytics_models', $model);
-
-        unset($model->contextids);
-        $DB->insert_record('analytics_models', $model);
-
-        $model->contextids = '0';
-        $DB->insert_record('analytics_models', $model);
-
-        $model->contextids = 'null';
-        $DB->insert_record('analytics_models', $model);
-
-        $select = $DB->sql_compare_text('contextids') . ' = :zero OR ' . $DB->sql_compare_text('contextids') . ' = :null';
-        $params = ['zero' => '0', 'null' => 'null'];
-        $this->assertEquals(2, $DB->count_records_select('analytics_models', $select, $params));
-
-        upgrade_analytics_fix_contextids_defaults();
-
-        $this->assertEquals(0, $DB->count_records_select('analytics_models', $select, $params));
-    }
-
     /**
      * Test the functionality of {@link upgrade_core_licenses} function.
      */
@@ -1118,7 +910,7 @@ class upgradelib_test extends advanced_testcase {
 
         upgrade_core_licenses();
 
-        $expectedshortnames = ['allrightsreserved', 'cc', 'cc-nc', 'cc-nc-nd', 'cc-nc-sa', 'cc-nd', 'cc-sa', 'public'];
+        $expectedshortnames = ['allrightsreserved', 'cc-4.0', 'cc-nc-4.0', 'cc-nc-nd-4.0', 'cc-nc-sa-4.0', 'cc-nd-4.0', 'cc-sa-4.0', 'public'];
         $licenses = $DB->get_records('license');
 
         foreach ($licenses as $license) {
@@ -1563,60 +1355,70 @@ class upgradelib_test extends advanced_testcase {
     }
 
     /**
-     * Test the check_xmlrpc_usage check when the MNet is turned on but no host was set up.
+     * Test the check_mod_assignment check if mod_assignment is still used.
      *
+     * @covers ::check_mod_assignment
      * @return void
      */
-    public function test_check_xmlrpc_mnet_host_is_not_set(): void {
-        global $CFG;
-
-        $this->resetAfterTest();
-        $CFG->mnet_dispatcher_mode = 'strict';
-
-        $result = new environment_results('custom_checks');
-        $this->assertNull(check_xmlrpc_usage($result));
-    }
-
-    /**
-     * Test the check_xmlrpc_usage check when the MNet is turned on and the host was set up.
-     *
-     * @return void
-     */
-    public function test_check_xmlrpc_mnet_host_is_set(): void {
+    public function test_check_mod_assignment_is_used(): void {
         global $CFG, $DB;
 
         $this->resetAfterTest();
-        $CFG->mnet_dispatcher_mode = 'strict';
-
-        // Add a mnet host.
-        $mnethost = new stdClass();
-        $mnethost->name = 'A mnet host';
-        $mnethost->public_key = 'A random public key!';
-        $mnethost->id = $DB->insert_record('mnet_host', $mnethost);
-
         $result = new environment_results('custom_checks');
-        $this->assertInstanceOf(environment_results::class, check_xmlrpc_usage($result));
-        $this->assertEquals('xmlrpc_mnet_usage', $result->getInfo());
-        $this->assertFalse($result->getStatus());
+
+        if (file_exists("{$CFG->dirroot}/mod/assignment/version.php")) {
+            // This is for when the test is run on sites where mod_assignment is most likely reinstalled.
+            $this->assertNull(check_mod_assignment($result));
+        } else {
+            // This is for when the test is run on sites with mod_assignment now gone.
+            $this->assertFalse($DB->get_manager()->table_exists('assignment'));
+            $this->assertNull(check_mod_assignment($result));
+
+            // Then we can simulate a scenario here where the assignment records are still present during the upgrade
+            // by recreating the assignment table and adding a record to it.
+            $dbman = $DB->get_manager();
+            $table = new xmldb_table('assignment');
+            $table->add_field('id', XMLDB_TYPE_INTEGER, '10', null, XMLDB_NOTNULL, XMLDB_SEQUENCE);
+            $table->add_field('name', XMLDB_TYPE_CHAR, '255');
+            $table->add_key('primary', XMLDB_KEY_PRIMARY, ['id']);
+            $dbman->create_table($table);
+            $DB->insert_record('assignment', (object)['name' => 'test_assign']);
+
+            $this->assertNotNull(check_mod_assignment($result));
+            $this->assertEquals('Assignment 2.2 is in use', $result->getInfo());
+            $this->assertFalse($result->getStatus());
+        }
     }
 
     /**
-     * Test the check_xmlrpc_usage check when the MNet is turned on and the Mahara portfolios was set up.
+     * Test the check_oracle_usage check when the Moodle instance is not using Oracle as a database architecture.
      *
-     * @return void
+     * @covers ::check_oracle_usage
      */
-    public function test_check_xmlrpc_mahara_portfolios_is_set(): void {
+    public function test_check_oracle_usage_is_not_used(): void {
         global $CFG;
 
         $this->resetAfterTest();
-        $CFG->mnet_dispatcher_mode = 'strict';
-
-        // Enable the Mahara portfolios.
-        \core\plugininfo\portfolio::enable_plugin('mahara', 1);
+        $CFG->dbtype = 'pgsql';
 
         $result = new environment_results('custom_checks');
-        $this->assertInstanceOf(environment_results::class, check_xmlrpc_usage($result));
-        $this->assertEquals('xmlrpc_mahara_usage', $result->getInfo());
+        $this->assertNull(check_oracle_usage($result));
+    }
+
+    /**
+     * Test the check_oracle_usage check when the Moodle instance is using Oracle as a database architecture.
+     *
+     * @covers ::check_oracle_usage
+     */
+    public function test_check_oracle_usage_is_used(): void {
+        global $CFG;
+
+        $this->resetAfterTest();
+        $CFG->dbtype = 'oci';
+
+        $result = new environment_results('custom_checks');
+        $this->assertInstanceOf(environment_results::class, check_oracle_usage($result));
+        $this->assertEquals('oracle_database_usage', $result->getInfo());
         $this->assertFalse($result->getStatus());
     }
 
@@ -1625,7 +1427,7 @@ class upgradelib_test extends advanced_testcase {
      *
      * @return array
      */
-    public function usermenu_items_dataprovider(): array {
+    public static function usermenu_items_dataprovider(): array {
         return [
             'Add new item to empty usermenu' => [
                 '',
@@ -1671,4 +1473,171 @@ calendar,core_calendar|/calendar/view.php?view=month',
 
         $this->assertEquals($expectedmenu, $newcustomusermenu);
     }
+
+    /**
+     * Test that file timestamps are corrected for copied files.
+     */
+    public function test_upgrade_fix_file_timestamps() {
+        global $DB;
+        $this->resetAfterTest();
+
+        // Add 2 files for testing, one with edited old timestamps.
+        $origtime = time();
+        $new = [
+            'contextid' => 123,
+            'component' => 'mod_label',
+            'filearea' => 'intro',
+            'itemid' => 0,
+            'filepath' => '/',
+            'filename' => 'file.txt',
+        ];
+        $old = [
+            'contextid' => 321,
+            'component' => 'mod_label',
+            'filearea' => 'intro',
+            'itemid' => 0,
+            'filepath' => '/',
+            'filename' => 'file.txt',
+        ];
+
+        // Create the file records. This will create a directory listing with the current time.
+        $fs = get_file_storage();
+        $newfile = $fs->create_file_from_string($new, 'new');
+        $oldfile = $fs->create_file_from_string($old, 'old');
+
+        // Manually set the timestamps to use on files.
+        $DB->set_field('files', 'timecreated', $origtime, [
+            'contextid' => $newfile->get_contextid(),
+            'component' => $newfile->get_component(),
+            'filearea' => $newfile->get_filearea(),
+            'itemid' => $newfile->get_itemid(),
+        ]);
+        $DB->set_field('files', 'timemodified', $origtime, [
+            'contextid' => $newfile->get_contextid(),
+            'component' => $newfile->get_component(),
+            'filearea' => $newfile->get_filearea(),
+            'itemid' => $newfile->get_itemid(),
+        ]);
+
+        $DB->set_field('files', 'timecreated', 1, ['id' => $oldfile->get_id()]);
+        $DB->set_field('files', 'timemodified', 1, ['id' => $oldfile->get_id()]);
+
+        upgrade_fix_file_timestamps();
+
+        // Check nothing changed on the new file.
+        $updatednew = $DB->get_record('files', ['id' => $newfile->get_id()]);
+        $this->assertEquals($origtime, $updatednew->timecreated);
+        $this->assertEquals($origtime, $updatednew->timemodified);
+
+        // Confirm that the file with old timestamps has been fixed.
+        $updatedold = $DB->get_record('files', ['id' => $oldfile->get_id()]);
+        $this->assertNotEquals(1, $updatedold->timecreated);
+        $this->assertNotEquals(1, $updatedold->timemodified);
+        $this->assertTrue($updatedold->timecreated >= $origtime);
+        $this->assertTrue($updatedold->timemodified >= $origtime);
+    }
+
+    /**
+     * Test the upgrade status check alongside the outageless flags.
+     *
+     * @covers ::moodle_needs_upgrading
+     */
+    public function test_moodle_upgrade_check_outageless() {
+        global $CFG;
+        $this->resetAfterTest();
+        // Get a baseline.
+        $this->assertFalse(moodle_needs_upgrading());
+
+        // First lets check a plain upgrade ready.
+        $CFG->version = '';
+        $this->assertTrue(moodle_needs_upgrading());
+
+        // Now set the locking config and confirm we shouldn't upgrade.
+        set_config('outagelessupgrade', true);
+        $this->assertFalse(moodle_needs_upgrading());
+
+        // Test the ignorelock flag is functioning.
+        $this->assertTrue(moodle_needs_upgrading(false));
+    }
+
+    /**
+     * Test the upgrade status check alongside the outageless flags.
+     *
+     * @covers ::upgrade_started
+     */
+    public function test_moodle_start_upgrade_outageless() {
+        global $CFG;
+        $this->resetAfterTest();
+        $this->assertObjectNotHasAttribute('upgraderunning', $CFG);
+
+        // Confirm that starting normally sets the upgraderunning flag.
+        upgrade_started();
+        $upgrade = get_config('core', 'upgraderunning');
+        $this->assertTrue($upgrade > (time() - 5));
+
+        // Confirm that the config flag doesnt affect the internal upgrade processes.
+        unset($CFG->upgraderunning);
+        set_config('upgraderunning', null);
+        set_config('outagelessupgrade', true);
+        upgrade_started();
+        $upgrade = get_config('core', 'upgraderunning');
+        $this->assertTrue($upgrade > (time() - 5));
+    }
+
+    /**
+     * Test the upgrade timeout setter alongside the outageless flags.
+     *
+     * @covers ::upgrade_set_timeout
+     */
+    public function test_moodle_set_upgrade_timeout_outageless() {
+        global $CFG;
+        $this->resetAfterTest();
+        $this->assertObjectNotHasAttribute('upgraderunning', $CFG);
+
+        // Confirm running normally sets the timeout.
+        upgrade_set_timeout(120);
+        $upgrade = get_config('core', 'upgraderunning');
+        $this->assertTrue($upgrade > (time() - 5));
+
+        // Confirm that the config flag doesnt affect the internal upgrade processes.
+        unset($CFG->upgraderunning);
+        set_config('upgraderunning', null);
+        set_config('outagelessupgrade', true);
+        upgrade_set_timeout(120);
+        $upgrade = get_config('core', 'upgraderunning');
+        $this->assertTrue($upgrade > (time() - 5));
+    }
+
+    /**
+     * Test the components of the upgrade process being run outageless.
+     *
+     * @covers ::moodle_needs_upgrading
+     * @covers ::upgrade_started
+     * @covers ::upgrade_set_timeout
+     */
+    public function test_upgrade_components_with_outageless() {
+        global $CFG;
+        $this->resetAfterTest();
+
+        // We can now define the outageless constant for use in upgrade, and test the effects.
+        define('CLI_UPGRADE_RUNNING', true);
+
+        // First test the upgrade check. Even when locked via config this should return true.
+        // This can happen when attempting to fix a broken upgrade, so needs to work.
+        set_config('outagelessupgrade', true);
+        $CFG->version = '';
+        $this->assertTrue(moodle_needs_upgrading());
+
+        // Now confirm that starting upgrade with the constant will not set the upgraderunning flag.
+        set_config('upgraderunning', null);
+        upgrade_started();
+        $upgrade = get_config('core', 'upgraderunning');
+        $this->assertFalse($upgrade);
+
+        // The same for timeouts, it should not be set if the constant is set.
+        set_config('upgraderunning', null);
+        upgrade_set_timeout(120);
+        $upgrade = get_config('core', 'upgraderunning');
+        $this->assertFalse($upgrade);
+    }
 }
