@@ -77,7 +77,7 @@ class meeting {
     /**
      * Get currently stored meeting info
      *
-     * @return mixed|stdClass
+     * @return stdClass
      */
     public function get_meeting_info() {
         if (!$this->meetinginfo) {
@@ -131,9 +131,9 @@ class meeting {
     /**
      * Get meeting attendees
      *
-     * @return mixed
+     * @return array[]
      */
-    public function get_attendees() {
+    public function get_attendees(): array {
         return $this->get_meeting_info()->attendees ?? [];
     }
 
@@ -147,12 +147,12 @@ class meeting {
     }
 
     /**
-     * Number of participants
+     * Total number of moderators and viewers.
      *
      * @return int
      */
     public function get_participant_count() {
-        return $this->get_meeting_info()->participantcount;
+        return $this->get_meeting_info()->totalusercount;
     }
 
     /**
@@ -166,7 +166,13 @@ class meeting {
         $presentation = $this->instance->get_presentation_for_bigbluebutton_upload(); // The URL must contain nonce.
         $presentationname = $presentation['name'] ?? null;
         $presentationurl = $presentation['url'] ?? null;
-        $response = bigbluebutton_proxy::create_meeting($data, $metadata, $presentationname, $presentationurl);
+        $response = bigbluebutton_proxy::create_meeting(
+            $data,
+            $metadata,
+            $presentationname,
+            $presentationurl,
+            $this->instance->get_instance_id()
+        );
         // New recording management: Insert a recordingID that corresponds to the meeting created.
         if ($this->instance->is_recorded()) {
             $recording = new recording(0, (object) [
@@ -184,7 +190,11 @@ class meeting {
      * Send an end meeting message to BBB server
      */
     public function end_meeting() {
-        bigbluebutton_proxy::end_meeting($this->instance->get_meeting_id(), $this->instance->get_moderator_password());
+        bigbluebutton_proxy::end_meeting(
+            $this->instance->get_meeting_id(),
+            $this->instance->get_moderator_password(),
+            $this->instance->get_instance_id()
+        );
     }
 
     /**
@@ -192,18 +202,21 @@ class meeting {
      *
      * @return string
      */
-    public function get_join_url() {
-        return bigbluebutton_proxy::get_join_url(
-            $this->instance->get_meeting_id(),
-            $this->instance->get_user_fullname(),
-            $this->instance->get_current_user_password(),
-            $this->instance->get_logout_url()->out(false),
-            null,
-            $this->instance->get_user_id(),
-            $this->get_meeting_info()->createtime
-        );
+    public function get_join_url(): string {
+        return bigbluebutton_proxy::get_join_url($this->instance, $this->get_meeting_info()->createtime);
     }
 
+    /**
+     * Get meeting join URL for guest
+     *
+     * @param string $userfullname
+     * @return string
+     */
+    public function get_guest_join_url(string $userfullname): string {
+        return bigbluebutton_proxy::get_guest_join_url($this->instance, $this->get_meeting_info()->createtime, $userfullname);
+    }
+
+
     /**
      * Return meeting information for this meeting.
      *
@@ -232,28 +245,27 @@ class meeting {
         $activitystatus = bigbluebutton_proxy::view_get_activity_status($instance);
         // This might raise an exception if info cannot be retrieved.
         // But this might be totally fine as the meeting is maybe not yet created on BBB side.
-        $participantcount = 0;
-        try {
-            $info = self::retrieve_cached_meeting_info($this->instance->get_meeting_id(), $updatecache);
+        $totalusercount = 0;
+        // This is the default value for any meeting that has not been created.
+        $meetinginfo->statusrunning = false;
+        $meetinginfo->createtime = null;
+
+        $info = self::retrieve_cached_meeting_info($this->instance, $updatecache);
+        if (!empty($info)) {
             $meetinginfo->statusrunning = $info['running'] === 'true';
             $meetinginfo->createtime = $info['createTime'] ?? null;
-            $participantcount = isset($info['participantCount']) ? $info['participantCount'] : 0;
-        } catch (bigbluebutton_exception $e) {
-            // The meeting is not created on BBB side, so we have to setup a couple of values here.
-            $meetinginfo->statusrunning = false;
-            $meetinginfo->createtime = null;
+            $totalusercount = isset($info['participantCount']) ? $info['participantCount'] : 0;
         }
+
         $meetinginfo->statusclosed = $activitystatus === 'ended';
         $meetinginfo->statusopen = !$meetinginfo->statusrunning && $activitystatus === 'open';
-        $meetinginfo->participantcount = $participantcount;
+        $meetinginfo->totalusercount = $totalusercount;
 
         $canjoin = !$instance->user_must_wait_to_join() || $meetinginfo->statusrunning;
-        // Limit has not been reached or user does not count toward limit.
-        $canjoin = $canjoin && (
-            !$instance->has_user_limit_been_reached($participantcount)
-            || !$instance->does_current_user_count_towards_user_limit()
-            );
-        $canjoin = $canjoin && ($instance->is_currently_open() || $instance->user_can_force_join());
+        // Limit has not been reached.
+        $canjoin = $canjoin && (!$instance->has_user_limit_been_reached($totalusercount));
+        // User should only join during scheduled session start and end time, if defined.
+        $canjoin = $canjoin && ($instance->is_currently_open());
         // Double check that the user has the capabilities to join.
         $canjoin = $canjoin && $instance->can_join();
         $meetinginfo->canjoin = $canjoin;
@@ -263,7 +275,7 @@ class meeting {
             $meetinginfo->startedat = floor(intval($info['startTime']) / 1000); // Milliseconds.
             $meetinginfo->moderatorcount = $info['moderatorCount'];
             $meetinginfo->moderatorplural = $info['moderatorCount'] > 1;
-            $meetinginfo->participantcount = $participantcount - $meetinginfo->moderatorcount;
+            $meetinginfo->participantcount = $totalusercount - $meetinginfo->moderatorcount;
             $meetinginfo->participantplural = $meetinginfo->participantcount > 1;
         }
         $meetinginfo->statusmessage = $this->get_status_message($meetinginfo, $instance);
@@ -274,11 +286,18 @@ class meeting {
         }
         $meetinginfo->attendees = [];
         if (!empty($info['attendees'])) {
-            // Make sure attendees is an array of object, not a simpleXML object.
+            // Ensure each returned attendee is cast to an array, rather than a simpleXML object.
             foreach ($info['attendees'] as $attendee) {
                 $meetinginfo->attendees[] = (array) $attendee;
             }
         }
+        $meetinginfo->guestaccessenabled = $instance->is_guest_allowed();
+        if ($meetinginfo->guestaccessenabled && $instance->is_moderator()) {
+            $meetinginfo->guestjoinurl = $instance->get_guest_access_url()->out();
+            $meetinginfo->guestpassword = $instance->get_guest_access_password();
+        }
+
+        $meetinginfo->features = $instance->get_enabled_features();
         return $meetinginfo;
     }
 
@@ -292,6 +311,9 @@ class meeting {
      * @return string
      */
     protected function get_status_message(object $meetinginfo, instance $instance): string {
+        if ($instance->has_user_limit_been_reached($meetinginfo->totalusercount)) {
+            return get_string('view_message_conference_user_limit_reached', 'bigbluebuttonbn');
+        }
         if ($meetinginfo->statusrunning) {
             return get_string('view_message_conference_in_progress', 'bigbluebuttonbn');
         }
@@ -310,12 +332,13 @@ class meeting {
     /**
      * Gets a meeting info object cached or fetched from the live session.
      *
-     * @param string $meetingid
+     * @param instance $instance
      * @param bool $updatecache
      *
      * @return array
      */
-    protected static function retrieve_cached_meeting_info($meetingid, $updatecache = false) {
+    protected static function retrieve_cached_meeting_info(instance $instance, $updatecache = false) {
+        $meetingid = $instance->get_meeting_id();
         $cachettl = (int) config::get('waitformoderator_cache_ttl');
         $cache = cache::make_from_params(cache_store::MODE_APPLICATION, 'mod_bigbluebuttonbn', 'meetings_cache');
         $result = $cache->get($meetingid);
@@ -324,11 +347,19 @@ class meeting {
             // Use the value in the cache.
             return (array) json_decode($result['meeting_info']);
         }
-        $cache->delete($meetingid); // Make sure we purges the cache before checking info.
+        // We set the cache to an empty value so then if get_meeting_info raises an exception we still have the
+        // info about the last creation_time, so we don't ask the server again for a bit.
+        $defaultcacheinfo = ['creation_time' => time(), 'meeting_info' => '[]'];
         // Pings again and refreshes the cache.
-        $meetinginfo = bigbluebutton_proxy::get_meeting_info($meetingid);
-
-        $cache->set($meetingid, ['creation_time' => time(), 'meeting_info' => json_encode($meetinginfo)]);
+        try {
+            $meetinginfo = bigbluebutton_proxy::get_meeting_info($meetingid);
+            $cache->set($meetingid, ['creation_time' => time(), 'meeting_info' => json_encode($meetinginfo)]);
+        } catch (bigbluebutton_exception $e) {
+            // The meeting is not created on BBB side, so we set the value in the cache so we don't poll again
+            // and return an empty array.
+            $cache->set($meetingid, $defaultcacheinfo);
+            return [];
+        }
         return $meetinginfo;
     }
 
@@ -340,12 +371,12 @@ class meeting {
         'disablemic' => 'lockSettingsDisableMic',
         'disableprivatechat' => 'lockSettingsDisablePrivateChat',
         'disablepublicchat' => 'lockSettingsDisablePublicChat',
-        'disablenote' => 'lockSettingsDisableNote',
-        'lockonjoin' => 'lockSettingsLockOnJoin',
+        'disablenote' => 'lockSettingsDisableNotes',
         'hideuserlist' => 'lockSettingsHideUserList'
     ];
     /**
      * Helper to prepare data used for create meeting.
+     * @todo moderatorPW and attendeePW will be removed from create after release of BBB v2.6.
      *
      * @return array
      */
@@ -377,11 +408,19 @@ class meeting {
         if ($this->instance->get_mute_on_start()) {
             $data['muteOnStart'] = 'true';
         }
+        // Here a bit of a change compared to the API default behaviour: we should not allow guest to join
+        // a meeting managed by Moodle by default.
+        if ($this->instance->is_guest_allowed()) {
+            $data['guestPolicy'] = $this->instance->is_moderator_approval_required() ? 'ASK_MODERATOR' : 'ALWAYS_ACCEPT';
+        }
         // Locks settings.
         foreach (self::LOCK_SETTINGS_MEETING_DATA as $instancevarname => $lockname) {
             $instancevar = $this->instance->get_instance_var($instancevarname);
             if (!is_null($instancevar)) {
                 $data[$lockname] = $instancevar ? 'true' : 'false';
+                if ($instancevar) {
+                    $data['lockSettingsLockOnJoin'] = 'true'; // This will be locked whenever one settings is locked.
+                }
             }
         }
         return $data;
@@ -426,7 +465,7 @@ class meeting {
             );
         }
         if ((boolean) config::get('recordingready_enabled')) {
-            $metadata['bn-recording-ready-url'] = $this->instance->get_record_ready_url()->out(false);
+            $metadata['bbb-recording-ready-url'] = $this->instance->get_record_ready_url()->out(false);
         }
         if ((boolean) config::get('meetingevents_enabled')) {
             $metadata['analytics-callback-url'] = $this->instance->get_meeting_event_notification_url()->out(false);
@@ -443,61 +482,31 @@ class meeting {
      *  - Body: <A JSON Object>
      *
      * @param instance $instance
-     * @return void
+     * @param object $data
+     * @return string
      */
-    public static function meeting_events(instance $instance) {
+    public static function meeting_events(instance $instance, object $data):  string {
         $bigbluebuttonbn = $instance->get_instance_data();
-        // Decodes the received JWT string.
-        try {
-            // Get the HTTP headers (getallheaders is a PHP function that may only work with Apache).
-            $headers = getallheaders();
-
-            // Pull the Bearer from the headers.
-            if (!array_key_exists('Authorization', $headers)) {
-                $msg = 'Authorization failed';
-                header('HTTP/1.0 400 Bad Request. ' . $msg);
-                return;
-            }
-            $authorization = explode(" ", $headers['Authorization']);
-
-            // Verify the authenticity of the request.
-            $token = \Firebase\JWT\JWT::decode(
-                $authorization[1],
-                new Key(config::get('shared_secret'), 'HS512')
-            );
-
-            // Get JSON string from the body.
-            $jsonstr = file_get_contents('php://input');
-
-            // Convert JSON string to a JSON object.
-            $jsonobj = json_decode($jsonstr);
-        } catch (Exception $e) {
-            $msg = 'Caught exception: ' . $e->getMessage();
-            header('HTTP/1.0 400 Bad Request. ' . $msg);
-            return;
-        }
-
         // Validate that the bigbluebuttonbn activity corresponds to the meeting_id received.
-        $meetingidelements = explode('[', $jsonobj->{'meeting_id'});
+        $meetingidelements = explode('[', $data->{'meeting_id'});
         $meetingidelements = explode('-', $meetingidelements[0]);
         if (!isset($bigbluebuttonbn) || $bigbluebuttonbn->meetingid != $meetingidelements[0]) {
-            $msg = 'The activity may have been deleted';
-            header('HTTP/1.0 410 Gone. ' . $msg);
-            return;
+            return 'HTTP/1.0 410 Gone. The activity may have been deleted';
         }
 
         // We make sure events are processed only once.
-        $overrides = ['meetingid' => $jsonobj->{'meeting_id'}];
-        $meta['recordid'] = $jsonobj->{'internal_meeting_id'};
+        $overrides = ['meetingid' => $data->{'meeting_id'}];
+        $meta['internalmeetingid'] = $data->{'internal_meeting_id'};
         $meta['callback'] = 'meeting_events';
+        $meta['meetingid'] = $data->{'meeting_id'};
 
         $eventcount = logger::log_event_callback($instance, $overrides, $meta);
         if ($eventcount === 1) {
             // Process the events.
-            self::process_meeting_events($instance, $jsonobj);
-            header('HTTP/1.0 200 Accepted. Enqueued.');
+            self::process_meeting_events($instance, $data);
+            return 'HTTP/1.0 200 Accepted. Enqueued.';
         } else {
-            header('HTTP/1.0 202 Accepted. Already processed.');
+            return 'HTTP/1.0 202 Accepted. Already processed.';
         }
     }
 
@@ -527,19 +536,15 @@ class meeting {
     }
 
     /**
-     * Join a meeting.
+     * Prepare join meeting action
      *
-     * @param int $origin The spec
-     * @return string The URL to redirect to
-     * @throws meeting_join_exception
+     * @param int $origin
+     * @return void
      */
-    public function join(int $origin): string {
+    protected function prepare_meeting_join_action(int $origin) {
         $this->do_get_meeting_info(true);
         if ($this->is_running()) {
-            if (
-                $this->instance->has_user_limit_been_reached($this->get_participant_count())
-                && $this->instance->does_current_user_count_towards_user_limit()
-            ) {
+            if ($this->instance->has_user_limit_been_reached($this->get_participant_count())) {
                 throw new meeting_join_exception('userlimitreached');
             }
         } else if ($this->instance->user_must_wait_to_join()) {
@@ -552,6 +557,29 @@ class meeting {
 
         // Before executing the redirect, increment the number of participants.
         roles::participant_joined($this->instance->get_meeting_id(), $this->instance->is_moderator());
+    }
+    /**
+     * Join a meeting.
+     *
+     * @param int $origin The spec
+     * @return string The URL to redirect to
+     * @throws meeting_join_exception
+     */
+    public function join(int $origin): string {
+        $this->prepare_meeting_join_action($origin);
+        return $this->get_join_url();
+    }
+
+    /**
+     * Join a meeting as a guest.
+     *
+     * @param int $origin The spec
+     * @param string $userfullname Fullname for the guest user
+     * @return string The URL to redirect to
+     * @throws meeting_join_exception
+     */
+    public function guest_join(int $origin, string $userfullname): string {
+        $this->prepare_meeting_join_action($origin);
         return $this->get_join_url();
     }
 }
