@@ -55,10 +55,12 @@ $string['addmultichoice'] = 'Create a Multichoice question page';
 $string['addnewgroupoverride'] = 'Add group override';
 $string['addnewuseroverride'] = 'Add user override';
 $string['addnumerical'] = 'Create a Numerical question page';
-$string['additionalattemptsremaining'] = 'Completed, You can re-attempt this lesson';
 $string['addpage'] = 'Add a page';
 $string['addshortanswer'] = 'Create a Short answer question page';
 $string['addtruefalse'] = 'Create a True/false question page';
+$string['allotheranswers'] = 'All other answers';
+$string['allotheranswersjump'] = 'All other answers jump';
+$string['allotheranswersscore'] = 'All other answers score';
 $string['allowofflineattempts'] = 'Allow lesson to be attempted offline using the mobile app';
 $string['allowofflineattempts_help'] = 'If enabled, a mobile app user can download the lesson and attempt it offline.
 All the possible answers and correct responses will be downloaded as well.
@@ -71,6 +73,8 @@ $string['answersfornumerical'] = 'Answers for numerical questions should be matc
 $string['arrangebuttonshorizontally'] = 'Arrange content buttons horizontally?';
 $string['attempt'] = 'Attempt: {$a}';
 $string['attemptheader'] = 'Attempt';
+$string['attemptinfonograde'] = '{$a->timestart} ({$a->duration})';
+$string['attemptinfowithgrade'] = '{$a->grade}% {$a->timestart} ({$a->duration})';
 $string['attempts'] = 'Attempts';
 $string['attemptsdeleted'] = 'Deleted attempts';
 $string['attemptsremaining'] = 'You have {$a} attempt(s) remaining';
@@ -79,6 +83,7 @@ $string['averagescore'] = 'Average score';
 $string['averagetime'] = 'Average time';
 $string['branch'] = 'Content';
 $string['branchtable'] = 'Content';
+$string['cachedef_overrides'] = 'User and group override information';
 $string['cancel'] = 'Cancel';
 $string['cannotfindanswer'] = 'Error: could not find answer';
 $string['cannotfindattempt'] = 'Error: could not find attempt';
@@ -109,9 +114,10 @@ $string['comments'] = 'Your comments';
 $string['completed'] = 'Completed';
 $string['completederror'] = 'Complete the lesson';
 $string['completethefollowingconditions'] = 'You must complete the following condition(s) in <b>{$a}</b> lesson before you can proceed.';
+$string['completiondetail:reachend'] = 'Go through the activity to the end';
+$string['completiondetail:timespent'] = 'Spend at least {$a} on this activity';
 $string['completionendreached'] = 'Require end reached';
 $string['completionendreached_desc'] = 'Student must reach the end of lesson page to complete this activity';
-$string['completiontimespent'] = 'Student must do this activity at least for';
 $string['completiontimespentdesc'] = 'Student must do this activity for at least {$a}';
 $string['completiontimespentgroup'] = 'Require time spent';
 $string['conditionsfordependency'] = 'Condition(s) for the dependency';
@@ -137,7 +143,7 @@ $string['correctresponse'] = 'Correct response';
 $string['createaquestionpage'] = 'Create a question page';
 $string['credit'] = 'Credit';
 $string['customscoring'] = 'Custom scoring';
-$string['customscoring_help'] = 'If enabled, then each answer may be given a numerical point value (positive or negative).';
+$string['customscoring_help'] = 'If enabled, a whole number value (positive or negative) may be entered for each answer.';
 $string['deadline'] = 'Deadline';
 $string['defaultessayresponse'] = 'Your essay will be graded by your teacher.';
 $string['deleteallattempts'] = 'Delete all lesson attempts';
@@ -149,6 +155,7 @@ $string['deletingpage'] = 'Deleting page: {$a}';
 $string['dependencyon'] = 'Dependent on';
 $string['dependencyon_help'] = 'This setting allows access to this lesson to be dependent upon a student\'s performance in another lesson in the same course. Any combination of time spent, completed or "grade better than" may be used.';
 $string['description'] = 'Description';
+$string['deselectallattempts'] = 'Deselect all attempts';
 $string['detailedstats'] = 'Detailed statistics';
 $string['didnotanswerquestion'] = 'Did not answer this question.';
 $string['didnotreceivecredit'] = 'Did not receive credit';
@@ -160,9 +167,10 @@ $string['displayleftif'] = 'Minimum grade to display menu';
 $string['displayleftif_help'] = 'This setting determines whether a student must obtain a certain grade before viewing the lesson menu. This forces the student to go through the entire lesson on their first attempt, then after obtaining the required grade they can use the menu for review.';
 $string['displayleftmenu'] = 'Display menu';
 $string['displayleftmenu_help'] = 'If enabled, a menu allowing users to navigate through the list of pages is displayed.';
+$string['displaymode'] = 'Display mode';
 $string['displayofgrade'] = 'Display of grade (for students only)';
 $string['displayreview'] = 'Provide option to try a question again';
-$string['displayreview_help'] = 'If enabled, when a question is answered incorrectly, the student is given the option to try it again for no point credit, or continue with the lesson.';
+$string['displayreview_help'] = 'If enabled, when a question is answered incorrectly, the student is given the option to try it again for no point credit, or continue with the lesson. If the student clicks to move on to another question then the selected (wrong) answer will be followed. By default wrong answer jumps are set to "this page" and have a score of 0, so it is recommended that you set the wrong answer jump to a different page to avoid confusion with your students.';
 $string['displayscorewithessays'] = '<p>You earned {$a->score} out of {$a->tempmaxgrade} for the automatically graded questions.</p>
 <p>Your {$a->essayquestions} essay question(s) will be graded and added into your final score at a later date.</p>
 <p>Your current grade without the essay question(s) is {$a->score} out of {$a->grade}.</p>';
@@ -175,6 +183,8 @@ $string['editendofcluster'] = 'Editing an end of cluster page';
 $string['editendofbranch'] = 'Editing an end of branch page';
 $string['editessay'] = 'Editing an Essay question page';
 $string['editingquestionpage'] = 'Editing {$a} question page';
+$string['editlesson'] = 'Edit lesson';
+$string['editinglesson'] = 'Editing lesson';
 $string['editlessonsettings'] = 'Edit lesson settings';
 $string['editmatching'] = 'Editing a Matching question page';
 $string['editmultichoice'] = 'Editing a Multichoice question page';
@@ -186,9 +196,9 @@ $string['editquestion'] = 'Editing a question page';
 $string['editshortanswer'] = 'Editing a Short answer question page';
 $string['edittruefalse'] = 'Editing a True/false question page';
 $string['email'] = 'Email';
-$string['emailallgradedessays'] = 'Email ALL graded essays';
-$string['emailgradedessays'] = 'Email graded essays';
-$string['emailsuccess'] = 'Emails sent successfully';
+$string['emailallgradedessays'] = 'Send essay graded notifications';
+$string['emailgradedessays'] = 'Send essay graded notifications';
+$string['emailsuccess'] = 'Notifications sent successfully';
 $string['enabled'] = 'Enabled';
 $string['endofbranch'] = 'End of branch';
 $string['endofcluster'] = 'End of cluster';
@@ -201,6 +211,7 @@ $string['eolstudentoutoftime'] = 'Attention:  You ran out of time for this lesso
 $string['eolstudentoutoftimenoanswers'] = 'You did not answer any questions.  You have received a 0 for this lesson.';
 $string['essay'] = 'Essay';
 $string['essayemailmessage2'] = '<p>Essay prompt: {$a->question}</p><p>Your response: <em>{$a->response}</em></p><p>Grader\'s comments: <em>{$a->comment}</em></p><p>You have received {$a->earned} out of {$a->outof} for this essay question.</p><p>Your grade for the {$a->lesson} lesson has been changed to {$a->newgrade}&#37;.</p>';
+$string['essayemailmessagesmall'] = '<p>You have received {$a->earned} out of {$a->outof} for this essay question.</p><p>Your grade for the {$a->lesson} lesson has been changed to {$a->newgrade}&#37;.</p>';
 $string['essayemailsubject'] = 'Grade available for lesson question';
 $string['essaynotgradedyet'] = 'This essay has not been graded yet';
 $string['essayresponses'] = 'Essay responses';
@@ -209,6 +220,8 @@ $string['essayscore'] = 'Essay score';
 $string['eventcontentpageviewed'] = 'Content page viewed';
 $string['eventessayassessed'] = 'Essay assessed';
 $string['eventessayattemptviewed'] = 'Essay attempt viewed';
+$string['eventhighscoreadded'] = 'Lesson high score added';
+$string['eventhighscoresviewed'] = 'Lesson high scores viewed';
 $string['eventlessonended'] = 'Lesson ended';
 $string['eventlessonrestarted'] = 'Lesson restarted';
 $string['eventlessonresumed'] = 'Lesson resumed';
@@ -257,12 +270,18 @@ $string['checkquestion'] = 'Check question';
 $string['importcount'] = 'Importing {$a} questions';
 $string['importquestions'] = 'Import questions';
 $string['importquestions_help'] = 'This feature enables questions in a variety of formats to be imported via text file.';
-$string['inactiveoverridehelp'] = '* Student does not have the correct group or role to view/attempt the lesson';
+$string['inactiveoverridehelp'] = '* This override is inactive because the user\'s access to the activity is restricted. This can be due to group or role assignments, other access restrictions, or the activity being hidden.';
 $string['insertedpage'] = 'Inserted page';
 $string['indicator:cognitivedepth'] = 'Lesson cognitive';
 $string['indicator:cognitivedepth_help'] = 'This indicator is based on the cognitive depth reached by the student in a Lesson activity.';
+$string['indicator:cognitivedepthdef'] = 'Lesson cognitive';
+$string['indicator:cognitivedepthdef_help'] = 'The participant has reached this percentage of the cognitive engagement offered by the Lesson activities during this analysis interval (Levels = No view, View, Submit, View feedback, Comment on feedback, Resubmit after viewing feedback)';
+$string['indicator:cognitivedepthdef_link'] = 'Learning_analytics_indicators#Cognitive_depth';
 $string['indicator:socialbreadth'] = 'Lesson social';
 $string['indicator:socialbreadth_help'] = 'This indicator is based on the social breadth reached by the student in a Lesson activity.';
+$string['indicator:socialbreadthdef'] = 'Lesson social';
+$string['indicator:socialbreadthdef_help'] = 'The participant has reached this percentage of the social engagement offered by the Lesson activities during this analysis interval (Levels = No participation, Participant alone, Participant with others)';
+$string['indicator:socialbreadthdef_link'] = 'Learning_analytics_indicators#Social_breadth';
 $string['invalidfile'] = 'Invalid file';
 $string['invalidid'] = 'No course module ID or lesson ID were passed';
 $string['invalidlessonid'] = 'lesson ID was incorrect';
@@ -278,16 +297,15 @@ $string['leftduringtimednoretake'] = 'You have left during a timed lesson and yo
 $string['lesson:addinstance'] = 'Add a new lesson';
 $string['lesson:grade'] = 'Grade lesson essay questions';
 $string['lessonclosed'] = 'This lesson closed on {$a}.';
+$string['lessonbeingpreviewed'] = 'Lesson is currently being previewed.';
 $string['lessoncloses'] = 'Lesson closes';
 $string['lessoneventcloses'] = '{$a} closes';
-$string['lessoncloseson'] = 'Lesson closes on {$a}';
 $string['lesson:edit'] = 'Edit a lesson activity';
 $string['lessonformating'] = 'Lesson formatting';
 $string['lesson:manage'] = 'Manage a lesson activity';
 $string['lesson:manageoverrides'] = 'Manage lesson overrides';
 $string['lesson:view'] = 'View lesson activity';
 $string['lesson:viewreports'] = 'View lesson reports';
-$string['lessonname'] = 'Lesson: {$a}';
 $string['lessonmenu'] = 'Lesson menu';
 $string['lessonnotready'] = 'This lesson is not ready to be taken.  Please contact your {$a}.';
 $string['lessonnotready2'] = 'This lesson is not ready to be taken.';
@@ -300,6 +318,7 @@ $string['linkedmedia'] = 'Linked media';
 $string['loginfail'] = 'Login failed, please try again...';
 $string['lowscore'] = 'Low score';
 $string['lowtime'] = 'Low time';
+$string['manageoverrides'] = 'Manage overrides';
 $string['manualgrading'] = 'Grade essays';
 $string['matchesanswer'] = 'Matches with answer';
 $string['matching'] = 'Matching';
@@ -308,7 +327,7 @@ $string['maxgrade'] = 'Maximum grade';
 $string['maxgrade_help'] = 'This setting specifies the maximum grade for the lesson. If set to 0, the lesson does not appear in the grades pages.';
 $string['maximumnumberofanswersbranches'] = 'Maximum number of answers';
 $string['maximumnumberofanswersbranches_help'] = 'This setting specifies the maximum number of answers that may be used in the lesson. If only true/false questions are used, it can be set to 2. The setting may be changed at any time, since it only affects what the teacher sees, not the data.';
-$string['maximumnumberofattempts'] = 'Maximum number of attempts';
+$string['maximumnumberofattempts'] = 'Maximum number of attempts per question';
 $string['maximumnumberofattempts_help'] = 'This setting specifies the maximum number of attempts allowed for each question. If answered incorrectly repeatedly, when the maximum is reached, the next page of the lesson is displayed.';
 $string['maximumnumberofattemptsreached'] = 'Maximum number of attempts reached - Moving to next page';
 $string['mediaclose'] = 'Show close button';
@@ -370,9 +389,11 @@ $string['noonehasansweredgroup'] = 'No one in {$a} has answered an essay questio
 $string['noonecheckedthis'] = 'No one checked this.';
 $string['noopen'] = 'No open date';
 $string['nooverridedata'] = 'You must override at least one of the lesson settings.';
+$string['nooverridecreated'] = 'No overrides created.';
 $string['noretake'] = 'You are not allowed to retake this lesson.';
 $string['normal'] = 'Normal - follow lesson path';
 $string['notcompleted'] = 'Not completed';
+$string['notcompletedwithdate'] = 'Not completed ({$a})';
 $string['notyetcompleted'] = 'Lesson has been started, but not yet completed';
 $string['notdefined'] = 'Not defined';
 $string['notenoughsubquestions'] = 'Not enough sub-questions have been defined!';
@@ -388,6 +409,8 @@ $string['numberofpagesviewed'] = 'Number of questions answered: {$a}';
 $string['numberofpagesviewedheader'] = 'Number of questions answered';
 $string['numberofpagesviewednotice'] = 'Number of questions answered: {$a->nquestions} (You should answer at least {$a->minquestions})';
 $string['numerical'] = 'Numerical';
+$string['numericanswer_help'] = 'You can specify a single number, or a range of numbers by using colon. For example 2:5 means any answer between 2 and 5 and including 2 and 5 is correct.';
+$string['numericanswer'] = 'Numeric answer';
 $string['offlinedatamessage'] = 'You have worked on this attempt using a mobile device. Data was last saved to this site {$a} ago. Please check that you do not have any unsaved work.';
 $string['ongoing'] = 'Display ongoing score';
 $string['ongoing_help'] = 'If enabled, each page will display the student\'s current points earned out of the total possible thus far.';
@@ -475,6 +498,7 @@ $string['privacy:metadata:overrides:password'] = 'The password to access the les
 $string['privacy:metadata:overrides'] = 'A record of overrides per lesson';
 $string['privacy:metadata:userpref:lessonview'] = 'The preferred display mode when editing lessons';
 $string['privacy:path:essayresponses'] = 'Essay responses';
+$string['privacy:path:essayanswers'] = 'Essay answers';
 $string['privacy:path:pages'] = 'Pages';
 $string['processerror'] = 'Error occurred during processing!';
 $string['progressbar'] = 'Progress bar';
@@ -513,11 +537,13 @@ $string['savechangesandeol'] = 'Save all changes and go to the end of the lesson
 $string['saveoverrideandstay'] = 'Save and enter another override';
 $string['savepage'] = 'Save page';
 $string['score'] = 'Score';
-$string['score_help'] = 'Score is only used when custom scoring is enabled. Each answer can then be given a numerical point value (positive or negative).';
+$string['score_help'] = 'If custom scoring is enabled in the lesson settings, a whole number value (positive or negative) may be entered for each answer.';
 $string['scores'] = 'Scores';
 $string['search:activity'] = 'Lesson - activity information';
 $string['secondpluswrong'] = 'Not quite.  Would you like to try again?';
 $string['selectaqtype'] = 'Select a question type';
+$string['selectallattempts'] = 'Select all attempts';
+$string['selectreport'] = 'Select report';
 $string['sent'] = 'Sent';
 $string['shortanswer'] = 'Short answer';
 $string['showanunansweredpage'] = 'Show an unanswered page';
@@ -576,13 +602,15 @@ $string['viewreports2'] = 'View {$a} completed attempts';
 $string['warning'] = 'Warning';
 $string['welldone'] = 'Well done!';
 $string['whatdofirst'] = 'What would you like to do first?';
+$string['withselectedattempts'] = 'With selected attempts...';
 $string['wronganswerjump'] = 'Wrong answer jump';
 $string['wronganswerscore'] = 'Wrong answer score';
 $string['wrongresponse'] = 'Wrong response';
-$string['xattempts'] = '{$a} attempts';
 $string['youhaveseen'] = 'You have seen more than one page of this lesson already.<br />Do you want to start at the last page you saw?';
 $string['youranswer'] = 'Your answer';
 $string['yourcurrentgradeis'] = 'Your current grade is {$a}';
 $string['yourcurrentgradeisoutof'] = 'Your current grade is {$a->grade} out of {$a->total}';
 $string['youshouldview'] = 'You should answer at least: {$a}';
 
+// Deprecated since 4.3.
+$string['completiontimespent'] = 'Student must do this activity at least for';
