@@ -14,16 +14,10 @@
 // You should have received a copy of the GNU General Public License
 // along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
 
-/**
- * Unit tests for the quizaccess_openclosedate plugin.
- *
- * @package    quizaccess
- * @subpackage openclosedate
- * @category   phpunit
- * @copyright  2008 The Open University
- * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
- */
+namespace quizaccess_openclosedate;
 
+use mod_quiz\quiz_settings;
+use quizaccess_openclosedate;
 
 defined('MOODLE_INTERNAL') || die();
 
@@ -34,19 +28,21 @@ require_once($CFG->dirroot . '/mod/quiz/accessrule/openclosedate/rule.php');
 /**
  * Unit tests for the quizaccess_openclosedate plugin.
  *
+ * @package    quizaccess_openclosedate
+ * @category   test
  * @copyright  2008 The Open University
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
-class quizaccess_openclosedate_testcase extends basic_testcase {
-    public function test_no_dates() {
-        $quiz = new stdClass();
+final class rule_test extends \basic_testcase {
+    public function test_no_dates(): void {
+        $quiz = new \stdClass();
         $quiz->timeopen = 0;
         $quiz->timeclose = 0;
         $quiz->overduehandling = 'autosubmit';
-        $cm = new stdClass();
+        $cm = new \stdClass();
         $cm->id = 0;
-        $quizobj = new quiz($quiz, $cm, null);
-        $attempt = new stdClass();
+        $quizobj = new quiz_settings($quiz, $cm, null);
+        $attempt = new \stdClass();
         $attempt->preview = 0;
 
         $rule = new quizaccess_openclosedate($quizobj, 10000);
@@ -66,14 +62,14 @@ class quizaccess_openclosedate_testcase extends basic_testcase {
     }
 
     public function test_start_date() {
-        $quiz = new stdClass();
+        $quiz = new \stdClass();
         $quiz->timeopen = 10000;
         $quiz->timeclose = 0;
         $quiz->overduehandling = 'autosubmit';
-        $cm = new stdClass();
+        $cm = new \stdClass();
         $cm->id = 0;
-        $quizobj = new quiz($quiz, $cm, null);
-        $attempt = new stdClass();
+        $quizobj = new quiz_settings($quiz, $cm, null);
+        $attempt = new \stdClass();
         $attempt->preview = 0;
 
         $rule = new quizaccess_openclosedate($quizobj, 9999);
@@ -93,14 +89,14 @@ class quizaccess_openclosedate_testcase extends basic_testcase {
     }
 
     public function test_close_date() {
-        $quiz = new stdClass();
+        $quiz = new \stdClass();
         $quiz->timeopen = 0;
         $quiz->timeclose = 20000;
         $quiz->overduehandling = 'autosubmit';
-        $cm = new stdClass();
+        $cm = new \stdClass();
         $cm->id = 0;
-        $quizobj = new quiz($quiz, $cm, null);
-        $attempt = new stdClass();
+        $quizobj = new quiz_settings($quiz, $cm, null);
+        $attempt = new \stdClass();
         $attempt->preview = 0;
 
         $rule = new quizaccess_openclosedate($quizobj, 20000);
@@ -127,14 +123,14 @@ class quizaccess_openclosedate_testcase extends basic_testcase {
     }
 
     public function test_both_dates() {
-        $quiz = new stdClass();
+        $quiz = new \stdClass();
         $quiz->timeopen = 10000;
         $quiz->timeclose = 20000;
         $quiz->overduehandling = 'autosubmit';
-        $cm = new stdClass();
+        $cm = new \stdClass();
         $cm->id = 0;
-        $quizobj = new quiz($quiz, $cm, null);
-        $attempt = new stdClass();
+        $quizobj = new quiz_settings($quiz, $cm, null);
+        $attempt = new \stdClass();
         $attempt->preview = 0;
 
         $rule = new quizaccess_openclosedate($quizobj, 9999);
@@ -167,15 +163,15 @@ class quizaccess_openclosedate_testcase extends basic_testcase {
     }
 
     public function test_close_date_with_overdue() {
-        $quiz = new stdClass();
+        $quiz = new \stdClass();
         $quiz->timeopen = 0;
         $quiz->timeclose = 20000;
         $quiz->overduehandling = 'graceperiod';
         $quiz->graceperiod = 1000;
-        $cm = new stdClass();
+        $cm = new \stdClass();
         $cm->id = 0;
-        $quizobj = new quiz($quiz, $cm, null);
-        $attempt = new stdClass();
+        $quizobj = new quiz_settings($quiz, $cm, null);
+        $attempt = new \stdClass();
         $attempt->preview = 0;
 
         $rule = new quizaccess_openclosedate($quizobj, 20000);
