@@ -69,7 +69,7 @@ class question_usage_by_activity {
     /** @var string plugin name of the plugin this usage belongs to. */
     protected $owningcomponent;
 
-    /** @var array {@link question_attempt}s that make up this usage. */
+    /** @var question_attempt[] {@link question_attempt}s that make up this usage. */
     protected $questionattempts = array();
 
     /** @var question_usage_observer that tracks changes to this usage. */
@@ -174,8 +174,10 @@ class question_usage_by_activity {
 
     /**
      * Add another question to this usage, in the place of an existing slot.
-     * The question_attempt that was in that slot is moved to the end at a new
-     * slot number, which is returned.
+     *
+     * Depending on $keepoldquestionattempt, the question_attempt that was in
+     * that slot is moved to the end at a new slot number, which is returned.
+     * Otherwise the existing attempt is completely removed and replaced.
      *
      * The added question is not started until you call {@link start_question()}
      * on it.
@@ -185,14 +187,18 @@ class question_usage_by_activity {
      * @param number $maxmark the maximum this question will be marked out of in
      *      this attempt (optional). If not given, the max mark from the $qa we
      *      are replacing is used.
+     * @param bool $keepoldquestionattempt if true (the default) we keep the existing
+     *      question_attempt, moving it to a new slot
      * @return int the new slot number of the question that was displaced.
      */
-    public function add_question_in_place_of_other($slot, question_definition $question, $maxmark = null) {
-        $newslot = $this->next_slot_number();
+    public function add_question_in_place_of_other(
+        $slot,
+        question_definition $question,
+        $maxmark = null,
+        bool $keepoldquestionattempt = true,
+    ) {
 
         $oldqa = $this->get_question_attempt($slot);
-        $oldqa->set_slot($newslot);
-        $this->questionattempts[$newslot] = $oldqa;
 
         if ($maxmark === null) {
             $maxmark = $oldqa->get_max_mark();
@@ -200,10 +206,26 @@ class question_usage_by_activity {
 
         $qa = new question_attempt($question, $this->get_id(), $this->observer, $maxmark);
         $qa->set_slot($slot);
-        $this->questionattempts[$slot] = $qa;
 
-        $this->observer->notify_attempt_moved($oldqa, $slot);
-        $this->observer->notify_attempt_added($qa);
+        if ($keepoldquestionattempt) {
+            $newslot = $this->next_slot_number();
+            $oldqa->set_slot($newslot);
+            $this->questionattempts[$newslot] = $oldqa;
+
+            $this->observer->notify_attempt_moved($oldqa, $slot);
+            $this->observer->notify_attempt_added($qa);
+
+        } else {
+            $newslot = $slot;
+            $qa->set_database_id($oldqa->get_database_id());
+
+            foreach ($oldqa->get_step_iterator() as $oldstep) {
+                $this->observer->notify_step_deleted($oldstep, $oldqa);
+            }
+            $this->observer->notify_attempt_modified($qa);
+        }
+
+        $this->questionattempts[$slot] = $qa;
 
         return $newslot;
     }
@@ -218,10 +240,12 @@ class question_usage_by_activity {
     /**
      * Get the question_definition for a question in this attempt.
      * @param int $slot the number used to identify this question within this usage.
+     * @param bool $requirequestioninitialised set this to false if you don't need
+     *      the behaviour initialised, which may improve performance.
      * @return question_definition the requested question object.
      */
-    public function get_question($slot) {
-        return $this->get_question_attempt($slot)->get_question();
+    public function get_question($slot, $requirequestioninitialised = true) {
+        return $this->get_question_attempt($slot)->get_question($requirequestioninitialised);
     }
 
     /** @return array all the identifying numbers of all the questions in this usage. */
@@ -701,7 +725,7 @@ class question_usage_by_activity {
             // Behaviour vars should not be processed by question type, just add prefix.
             $behaviourvars = $this->get_question_attempt($slot)->get_behaviour()->get_expected_data();
             foreach (array_keys($responsedata) as $responsedatakey) {
-                if ($responsedatakey[0] === '-') {
+                if (is_string($responsedatakey) && $responsedatakey[0] === '-') {
                     $behaviourvarname = substr($responsedatakey, 1);
                     if (isset($behaviourvars[$behaviourvarname])) {
                         // Expected behaviour var found.
@@ -873,22 +897,41 @@ class question_usage_by_activity {
         $this->observer->notify_attempt_modified($qa);
     }
 
+    /**
+     * Verify if the question_attempt in the given slot can be regraded with that other question version.
+     *
+     * @param int $slot the number used to identify this question within this usage.
+     * @param question_definition $otherversion a different version of the question to use in the regrade.
+     * @return string|null null if the regrade can proceed, else a reason why not.
+     */
+    public function validate_can_regrade_with_other_version(int $slot, question_definition $otherversion): ?string {
+        return $this->get_question_attempt($slot)->validate_can_regrade_with_other_version($otherversion);
+    }
+
     /**
      * Regrade a question in this usage. This replays the sequence of submitted
      * actions to recompute the outcomes.
+     *
      * @param int $slot the number used to identify this question within this usage.
      * @param bool $finished whether the question attempt should be forced to be finished
      *      after the regrade, or whether it may still be in progress (default false).
      * @param number $newmaxmark (optional) if given, will change the max mark while regrading.
+     * @param question_definition|null $otherversion a different version of the question to use
+     *      in the regrade. (By default, the regrode will use exactly the same question version.)
      */
-    public function regrade_question($slot, $finished = false, $newmaxmark = null) {
+    public function regrade_question($slot, $finished = false, $newmaxmark = null,
+            question_definition $otherversion = null) {
         $oldqa = $this->get_question_attempt($slot);
+        if ($otherversion &&
+                $otherversion->questionbankentryid !== $oldqa->get_question(false)->questionbankentryid) {
+            throw new coding_exception('You can only regrade using a different version of the same question, ' .
+                    'not a completely different question.');
+        }
         if (is_null($newmaxmark)) {
             $newmaxmark = $oldqa->get_max_mark();
         }
-
-        $newqa = new question_attempt($oldqa->get_question(), $oldqa->get_usage_id(),
-                $this->observer, $newmaxmark);
+        $newqa = new question_attempt($otherversion ?? $oldqa->get_question(false),
+                $oldqa->get_usage_id(), $this->observer, $newmaxmark);
         $newqa->set_database_id($oldqa->get_database_id());
         $newqa->set_slot($oldqa->get_slot());
         $newqa->regrade($oldqa, $finished);
@@ -965,6 +1008,34 @@ class question_usage_by_activity {
 
         return $quba;
     }
+
+    /**
+     * Preload users of all question attempt steps.
+     *
+     * @throws dml_exception
+     */
+    public function preload_all_step_users(): void {
+        global $DB;
+
+        // Get all user ids.
+        $userids = [];
+        foreach ($this->questionattempts as $qa) {
+            foreach ($qa->get_full_step_iterator() as $step) {
+                $userids[$step->get_user_id()] = 1;
+            }
+        }
+
+        // Load user information.
+        $users = $DB->get_records_list('user', 'id', array_keys($userids), '', '*');
+        // Update user information for steps.
+        foreach ($this->questionattempts as $qa) {
+            foreach ($qa->get_full_step_iterator() as $step) {
+                if (isset($users[$step->get_user_id()])) {
+                    $step->add_full_user_object($users[$step->get_user_id()]);
+                }
+            }
+        }
+    }
 }
 
 
@@ -1006,6 +1077,7 @@ class question_attempt_iterator implements Iterator, ArrayAccess {
      *
      * @return question_attempt
      */
+    #[\ReturnTypeWillChange]
     public function current() {
         return $this->offsetGet(current($this->slots));
     }
@@ -1015,6 +1087,7 @@ class question_attempt_iterator implements Iterator, ArrayAccess {
      *
      * @return int
      */
+    #[\ReturnTypeWillChange]
     public function key() {
         return current($this->slots);
     }
@@ -1022,14 +1095,14 @@ class question_attempt_iterator implements Iterator, ArrayAccess {
     /**
      * Standard part of the Iterator interface.
      */
-    public function next() {
+    public function next(): void {
         next($this->slots);
     }
 
     /**
      * Standard part of the Iterator interface.
      */
-    public function rewind() {
+    public function rewind(): void {
         reset($this->slots);
     }
 
@@ -1038,7 +1111,7 @@ class question_attempt_iterator implements Iterator, ArrayAccess {
      *
      * @return bool
      */
-    public function valid() {
+    public function valid(): bool {
         return current($this->slots) !== false;
     }
 
@@ -1048,7 +1121,7 @@ class question_attempt_iterator implements Iterator, ArrayAccess {
      * @param int $slot
      * @return bool
      */
-    public function offsetExists($slot) {
+    public function offsetExists($slot): bool {
         return in_array($slot, $this->slots);
     }
 
@@ -1058,6 +1131,7 @@ class question_attempt_iterator implements Iterator, ArrayAccess {
      * @param int $slot
      * @return question_attempt
      */
+    #[\ReturnTypeWillChange]
     public function offsetGet($slot) {
         return $this->quba->get_question_attempt($slot);
     }
@@ -1068,7 +1142,7 @@ class question_attempt_iterator implements Iterator, ArrayAccess {
      * @param int $slot
      * @param question_attempt $value
      */
-    public function offsetSet($slot, $value) {
+    public function offsetSet($slot, $value): void {
         throw new coding_exception('You are only allowed read-only access to ' .
                 'question_attempt::states through a question_attempt_step_iterator. Cannot set.');
     }
@@ -1078,7 +1152,7 @@ class question_attempt_iterator implements Iterator, ArrayAccess {
      *
      * @param int $slot
      */
-    public function offsetUnset($slot) {
+    public function offsetUnset($slot): void {
         throw new coding_exception('You are only allowed read-only access to ' .
                 'question_attempt::states through a question_attempt_step_iterator. Cannot unset.');
     }
